@@ -1,12 +1,13 @@
 /*
    Directory cache support
 
-   Copyright (C) 1998-2019
+   Copyright (C) 1998-2024
    Free Software Foundation, Inc.
 
    Written by:
    Pavel Machek <pavel@ucw.cz>, 1998
-   Slava Zanko <slavazanko@gmail.com>, 2013
+   Slava Zanko <slavazanko@gmail.com>, 2010-2013
+   Andrew Borodin <aborodin@vmail.ru> 2010-2022
 
    This file is part of the Midnight Commander.
 
@@ -58,15 +59,18 @@
 #include <config.h>
 
 #include <errno.h>
-#include <time.h>
-#include <sys/time.h>           /* gettimeofday() */
 #include <inttypes.h>           /* uintmax_t */
 #include <stdarg.h>
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+#include <sys/types.h>
+#include <unistd.h>
 
 #include "lib/global.h"
 
 #include "lib/tty/tty.h"        /* enable/disable interrupt key */
-#include "lib/util.h"           /* custom_canonicalize_pathname() */
+#include "lib/util.h"           /* canonicalize_pathname_custom() */
 #if 0
 #include "lib/widget.h"         /* message() */
 #endif
@@ -94,6 +98,8 @@ struct dirhandle
 
 /*** file scope variables ************************************************************************/
 
+/*** forward declarations (file scope functions) *************************************************/
+
 /* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
@@ -178,7 +184,7 @@ vfs_s_find_entry_tree (struct vfs_class *me, struct vfs_s_inode *root,
     char *path = pathref;
 
     /* canonicalize as well, but don't remove '../' from path */
-    custom_canonicalize_pathname (path, CANON_PATH_ALL & (~CANON_PATH_REMDOUBLEDOTS));
+    canonicalize_pathname_custom (path, CANON_PATH_ALL & (~CANON_PATH_REMDOUBLEDOTS));
 
     while (root != NULL)
     {
@@ -196,7 +202,7 @@ vfs_s_find_entry_tree (struct vfs_class *me, struct vfs_s_inode *root,
         for (pseg = 0; path[pseg] != '\0' && !IS_PATH_SEP (path[pseg]); pseg++)
             ;
 
-        for (iter = root->subdir; iter != NULL; iter = g_list_next (iter))
+        for (iter = g_queue_peek_head_link (root->subdir); iter != NULL; iter = g_list_next (iter))
         {
             ent = VFS_ENTRY (iter->data);
             if (strlen (ent->name) == pseg && strncmp (ent->name, path, pseg) == 0)
@@ -242,7 +248,7 @@ vfs_s_find_entry_linear (struct vfs_class *me, struct vfs_s_inode *root,
         vfs_die ("We have to use _real_ root. Always. Sorry.");
 
     /* canonicalize as well, but don't remove '../' from path */
-    custom_canonicalize_pathname (path, CANON_PATH_ALL & (~CANON_PATH_REMDOUBLEDOTS));
+    canonicalize_pathname_custom (path, CANON_PATH_ALL & (~CANON_PATH_REMDOUBLEDOTS));
 
     if ((flags & FL_DIR) == 0)
     {
@@ -259,7 +265,7 @@ vfs_s_find_entry_linear (struct vfs_class *me, struct vfs_s_inode *root,
         return ent;
     }
 
-    iter = g_list_find_custom (root->subdir, path, (GCompareFunc) vfs_s_entry_compare);
+    iter = g_queue_find_custom (root->subdir, path, (GCompareFunc) vfs_s_entry_compare);
     ent = iter != NULL ? VFS_ENTRY (iter->data) : NULL;
 
     if (ent != NULL && !VFS_SUBCLASS (me)->dir_uptodate (me, ent->ino))
@@ -286,7 +292,7 @@ vfs_s_find_entry_linear (struct vfs_class *me, struct vfs_s_inode *root,
 
         vfs_s_insert_entry (me, root, ent);
 
-        iter = g_list_find_custom (root->subdir, path, (GCompareFunc) vfs_s_entry_compare);
+        iter = g_queue_find_custom (root->subdir, path, (GCompareFunc) vfs_s_entry_compare);
         ent = iter != NULL ? VFS_ENTRY (iter->data) : NULL;
     }
     if (ent == NULL)
@@ -337,7 +343,7 @@ vfs_s_free_super (struct vfs_class *me, struct vfs_s_super *super)
     }
 
 #if 0
-    /* FIXME: We currently leak small ammount of memory, sometimes. Fix it if you can. */
+    /* FIXME: We currently leak small amount of memory, sometimes. Fix it if you can. */
     if (super->ino_usage != 0)
         message (D_ERROR, "Direntry warning",
                  "Super ino_usage is %d, memory leak", super->ino_usage);
@@ -390,22 +396,22 @@ vfs_s_inode_from_path (const vfs_path_t * vpath, int flags)
     struct vfs_s_super *super;
     struct vfs_s_inode *ino;
     const char *q;
-    const vfs_path_element_t *path_element;
+    struct vfs_class *me;
 
     q = vfs_s_get_path (vpath, &super, 0);
     if (q == NULL)
         return NULL;
 
-    path_element = vfs_path_get_by_index (vpath, -1);
+    me = VFS_CLASS (vfs_path_get_last_path_vfs (vpath));
 
     ino =
-        vfs_s_find_inode (path_element->class, super, q,
+        vfs_s_find_inode (me, super, q,
                           (flags & FL_FOLLOW) != 0 ? LINK_FOLLOW : LINK_NO_FOLLOW,
                           flags & ~FL_FOLLOW);
     if (ino == NULL && *q == '\0')
         /* We are asking about / directory of ftp server: assume it exists */
         ino =
-            vfs_s_find_inode (path_element->class, super, q,
+            vfs_s_find_inode (me, super, q,
                               (flags & FL_FOLLOW) != 0 ? LINK_FOLLOW : LINK_NO_FOLLOW,
                               FL_DIR | (flags & ~FL_FOLLOW));
     return ino;
@@ -418,17 +424,17 @@ vfs_s_opendir (const vfs_path_t * vpath)
 {
     struct vfs_s_inode *dir;
     struct dirhandle *info;
-    const vfs_path_element_t *path_element;
+    struct vfs_class *me;
 
     dir = vfs_s_inode_from_path (vpath, FL_DIR | FL_FOLLOW);
     if (dir == NULL)
         return NULL;
 
-    path_element = vfs_path_get_by_index (vpath, -1);
+    me = VFS_CLASS (vfs_path_get_last_path_vfs (vpath));
 
     if (!S_ISDIR (dir->st.st_mode))
     {
-        path_element->class->verrno = ENOTDIR;
+        me->verrno = ENOTDIR;
         return NULL;
     }
 
@@ -436,12 +442,12 @@ vfs_s_opendir (const vfs_path_t * vpath)
 #if 0
     if (dir->subdir == NULL)    /* This can actually happen if we allow empty directories */
     {
-        path_element->class->verrno = EAGAIN;
+        me->verrno = EAGAIN;
         return NULL;
     }
 #endif
     info = g_new (struct dirhandle, 1);
-    info->cur = dir->subdir;
+    info->cur = g_queue_peek_head_link (dir->subdir);
     info->dir = dir;
 
     return info;
@@ -449,10 +455,10 @@ vfs_s_opendir (const vfs_path_t * vpath)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static void *
+static struct vfs_dirent *
 vfs_s_readdir (void *data)
 {
-    static union vfs_dirent dir;
+    struct vfs_dirent *dir = NULL;
     struct dirhandle *info = (struct dirhandle *) data;
     const char *name;
 
@@ -461,13 +467,13 @@ vfs_s_readdir (void *data)
 
     name = VFS_ENTRY (info->cur->data)->name;
     if (name != NULL)
-        g_strlcpy (dir.dent.d_name, name, MC_MAXPATHLEN);
+        dir = vfs_dirent_init (NULL, name, 0);
     else
         vfs_die ("Null in structure-cannot happen");
 
     info->cur = g_list_next (info->cur);
 
-    return (void *) &dir;
+    return dir;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -519,23 +525,23 @@ vfs_s_readlink (const vfs_path_t * vpath, char *buf, size_t size)
 {
     struct vfs_s_inode *ino;
     size_t len;
-    const vfs_path_element_t *path_element;
+    struct vfs_class *me;
 
     ino = vfs_s_inode_from_path (vpath, 0);
     if (ino == NULL)
         return (-1);
 
-    path_element = vfs_path_get_by_index (vpath, -1);
+    me = VFS_CLASS (vfs_path_get_last_path_vfs (vpath));
 
     if (!S_ISLNK (ino->st.st_mode))
     {
-        path_element->class->verrno = EINVAL;
+        me->verrno = EINVAL;
         return (-1);
     }
 
     if (ino->linkname == NULL)
     {
-        path_element->class->verrno = EFAULT;
+        me->verrno = EFAULT;
         return (-1);
     }
 
@@ -668,7 +674,7 @@ vfs_s_close (void *fh)
         sub->linear_close (me, fh);
     if (sub->fh_close != NULL)
         res = sub->fh_close (me, fh);
-    if ((me->flags & VFS_USETMP) != 0 && file->changed && sub->file_store != NULL)
+    if ((me->flags & VFSF_USETMP) != 0 && file->changed && sub->file_store != NULL)
     {
         char *s;
 
@@ -759,8 +765,8 @@ vfs_s_getlocalcopy (const vfs_path_t * vpath)
     {
         const struct vfs_class *me;
 
-        me = vfs_path_get_by_index (vpath, -1)->class;
-        if ((me->flags & VFS_USETMP) != 0 && fh->ino != NULL)
+        me = vfs_path_get_last_path_vfs (vpath);
+        if ((me->flags & VFSF_USETMP) != 0 && fh->ino != NULL)
             local = vfs_path_from_str_flags (fh->ino->localname, VPF_NO_CANON);
 
         vfs_s_close (fh);
@@ -789,9 +795,9 @@ vfs_s_ungetlocalcopy (const vfs_path_t * vpath, const vfs_path_t * local, gboole
 static int
 vfs_s_setctl (const vfs_path_t * vpath, int ctlop, void *arg)
 {
-    const vfs_path_element_t *path_element;
+    struct vfs_class *me;
 
-    path_element = vfs_path_get_by_index (vpath, -1);
+    me = VFS_CLASS (vfs_path_get_last_path_vfs (vpath));
 
     switch (ctlop)
     {
@@ -807,15 +813,15 @@ vfs_s_setctl (const vfs_path_t * vpath, int ctlop, void *arg)
             else
             {
                 ino->super->want_stale = FALSE;
-                vfs_s_invalidate (path_element->class, ino->super);
+                vfs_s_invalidate (me, ino->super);
             }
             return 1;
         }
     case VFS_SETCTL_LOGFILE:
-        path_element->class->logfile = fopen ((char *) arg, "w");
+        me->logfile = fopen ((char *) arg, "w");
         return 1;
     case VFS_SETCTL_FLUSH:
-        path_element->class->flush = TRUE;
+        me->flush = TRUE;
         return 1;
     default:
         return 0;
@@ -856,10 +862,10 @@ vfs_s_free (vfsid id)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
+static gboolean
 vfs_s_dir_uptodate (struct vfs_class *me, struct vfs_s_inode *ino)
 {
-    struct timeval tim;
+    gint64 tim;
 
     if (me->flush)
     {
@@ -867,9 +873,9 @@ vfs_s_dir_uptodate (struct vfs_class *me, struct vfs_s_inode *ino)
         return 0;
     }
 
-    gettimeofday (&tim, NULL);
+    tim = g_get_monotonic_time ();
 
-    return (tim.tv_sec < ino->timestamp.tv_sec ? 1 : 0);
+    return (tim < ino->timestamp);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -888,6 +894,7 @@ vfs_s_new_inode (struct vfs_class *me, struct vfs_s_super *super, struct stat *i
     if (initstat != NULL)
         ino->st = *initstat;
     ino->super = super;
+    ino->subdir = g_queue_new ();
     ino->st.st_nlink = 0;
     ino->st.st_ino = VFS_SUBCLASS (me)->inode_counter++;
     ino->st.st_dev = VFS_SUBCLASS (me)->rdev;
@@ -914,12 +921,20 @@ vfs_s_free_inode (struct vfs_class *me, struct vfs_s_inode *ino)
         return;
     }
 
-    while (ino->subdir != NULL)
-        vfs_s_free_entry (me, VFS_ENTRY (ino->subdir->data));
+    while (g_queue_get_length (ino->subdir) != 0)
+    {
+        struct vfs_s_entry *entry;
+
+        entry = VFS_ENTRY (g_queue_peek_head (ino->subdir));
+        vfs_s_free_entry (me, entry);
+    }
+
+    g_queue_free (ino->subdir);
+    ino->subdir = NULL;
 
     CALL (free_inode) (me, ino);
     g_free (ino->linkname);
-    if ((me->flags & VFS_USETMP) != 0 && ino->localname != NULL)
+    if ((me->flags & VFSF_USETMP) != 0 && ino->localname != NULL)
     {
         unlink (ino->localname);
         g_free (ino->localname);
@@ -951,7 +966,7 @@ void
 vfs_s_free_entry (struct vfs_class *me, struct vfs_s_entry *ent)
 {
     if (ent->dir != NULL)
-        ent->dir->subdir = g_list_remove (ent->dir->subdir, ent);
+        g_queue_remove (ent->dir->subdir, ent);
 
     MC_PTR_FREE (ent->name);
 
@@ -974,7 +989,7 @@ vfs_s_insert_entry (struct vfs_class *me, struct vfs_s_inode *dir, struct vfs_s_
     ent->dir = dir;
 
     ent->ino->st.st_nlink++;
-    dir->subdir = g_list_append (dir->subdir, ent);
+    g_queue_push_tail (dir->subdir, ent);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1082,7 +1097,7 @@ vfs_s_find_inode (struct vfs_class *me, const struct vfs_s_super *super,
 {
     struct vfs_s_entry *ent;
 
-    if (((me->flags & VFS_REMOTE) == 0) && (*path == '\0'))
+    if (((me->flags & VFSF_REMOTE) == 0) && (*path == '\0'))
         return super->root;
 
     ent = VFS_SUBCLASS (me)->find_entry (me, super->root, path, follow, flags);
@@ -1142,7 +1157,7 @@ vfs_get_super_by_vpath (const vfs_path_t * vpath)
     }
 
   ret:
-    vfs_path_free (vpath_archive);
+    vfs_path_free (vpath_archive, TRUE);
     return super;
 }
 
@@ -1193,7 +1208,7 @@ vfs_s_get_path (const vfs_path_t * vpath, struct vfs_s_super **archive, int flag
         vfs_path_remove_element_by_index (vpath_archive, -1);
 
         result = subclass->open_archive (super, vpath_archive, path_element);
-        vfs_path_free (vpath_archive);
+        vfs_path_free (vpath_archive, TRUE);
     }
     if (result == -1)
     {
@@ -1234,7 +1249,7 @@ vfs_s_fullpath (struct vfs_class *me, struct vfs_s_inode *ino)
     if (ino->ent == NULL)
         ERRNOR (EAGAIN, NULL);
 
-    if ((me->flags & VFS_USETMP) == 0)
+    if ((me->flags & VFSF_USETMP) == 0)
     {
         /* archives */
         char *path;
@@ -1285,72 +1300,67 @@ vfs_s_open (const vfs_path_t * vpath, int flags, mode_t mode)
     struct vfs_s_super *super;
     const char *q;
     struct vfs_s_inode *ino;
-    const vfs_path_element_t *path_element;
+    struct vfs_class *me;
     struct vfs_s_subclass *s;
 
     q = vfs_s_get_path (vpath, &super, 0);
     if (q == NULL)
         return NULL;
 
-    path_element = vfs_path_get_by_index (vpath, -1);
+    me = VFS_CLASS (vfs_path_get_last_path_vfs (vpath));
 
-    ino = vfs_s_find_inode (path_element->class, super, q, LINK_FOLLOW, FL_NONE);
+    ino = vfs_s_find_inode (me, super, q, LINK_FOLLOW, FL_NONE);
     if (ino != NULL && (flags & (O_CREAT | O_EXCL)) == (O_CREAT | O_EXCL))
     {
-        path_element->class->verrno = EEXIST;
+        me->verrno = EEXIST;
         return NULL;
     }
 
-    s = VFS_SUBCLASS (path_element->class);
+    s = VFS_SUBCLASS (me);
 
     if (ino == NULL)
     {
-        char *dirname, *name;
+        char *name;
         struct vfs_s_entry *ent;
         struct vfs_s_inode *dir;
 
         /* If the filesystem is read-only, disable file creation */
-        if ((flags & O_CREAT) == 0 || path_element->class->write == NULL)
+        if ((flags & O_CREAT) == 0 || me->write == NULL)
             return NULL;
 
-        dirname = g_path_get_dirname (q);
-        name = g_path_get_basename (q);
-        dir = vfs_s_find_inode (path_element->class, super, dirname, LINK_FOLLOW, FL_DIR);
+        name = g_path_get_dirname (q);
+        dir = vfs_s_find_inode (me, super, name, LINK_FOLLOW, FL_DIR);
+        g_free (name);
         if (dir == NULL)
-        {
-            g_free (dirname);
-            g_free (name);
             return NULL;
-        }
 
-        ent = vfs_s_generate_entry (path_element->class, name, dir, 0755);
+        name = g_path_get_basename (q);
+        ent = vfs_s_generate_entry (me, name, dir, 0755);
         ino = ent->ino;
-        vfs_s_insert_entry (path_element->class, dir, ent);
-        if ((VFS_CLASS (s)->flags & VFS_USETMP) != 0)
+        vfs_s_insert_entry (me, dir, ent);
+        if ((VFS_CLASS (s)->flags & VFSF_USETMP) != 0)
         {
             int tmp_handle;
             vfs_path_t *tmp_vpath;
 
-            tmp_handle = vfs_mkstemps (&tmp_vpath, path_element->class->name, name);
-            ino->localname = g_strdup (vfs_path_as_str (tmp_vpath));
-            vfs_path_free (tmp_vpath);
+            tmp_handle = vfs_mkstemps (&tmp_vpath, me->name, name);
+            ino->localname = vfs_path_free (tmp_vpath, FALSE);
             if (tmp_handle == -1)
             {
-                g_free (dirname);
                 g_free (name);
                 return NULL;
             }
 
             close (tmp_handle);
         }
-        g_free (dirname);
+
         g_free (name);
         was_changed = TRUE;
     }
 
     if (S_ISDIR (ino->st.st_mode))
     {
-        path_element->class->verrno = EISDIR;
+        me->verrno = EISDIR;
         return NULL;
     }
 
@@ -1366,26 +1376,26 @@ vfs_s_open (const vfs_path_t * vpath, int flags, mode_t mode)
     }
     else
     {
-        if (s->fh_open != NULL && s->fh_open (path_element->class, fh, flags, mode) != 0)
+        if (s->fh_open != NULL && s->fh_open (me, fh, flags, mode) != 0)
         {
             vfs_s_free_fh (s, fh);
             return NULL;
         }
     }
 
-    if ((VFS_CLASS (s)->flags & VFS_USETMP) != 0 && fh->ino->localname != NULL)
+    if ((VFS_CLASS (s)->flags & VFSF_USETMP) != 0 && fh->ino->localname != NULL)
     {
         fh->handle = open (fh->ino->localname, NO_LINEAR (flags), mode);
         if (fh->handle == -1)
         {
             vfs_s_free_fh (s, fh);
-            path_element->class->verrno = errno;
+            me->verrno = errno;
             return NULL;
         }
     }
 
     /* i.e. we had no open files and now we have one */
-    vfs_rmstamp (path_element->class, (vfsid) super);
+    vfs_rmstamp (me, (vfsid) super);
     super->fd_usage++;
     fh->ino->st.st_nlink++;
     return fh;
@@ -1423,7 +1433,7 @@ vfs_s_retrieve_file (struct vfs_class *me, struct vfs_s_inode *ino)
 {
     /* If you want reget, you'll have to open file with O_LINEAR */
     off_t total = 0;
-    char buffer[8192];
+    char buffer[BUF_8K];
     int handle;
     ssize_t n;
     off_t stat_size = ino->st.st_size;
@@ -1431,12 +1441,11 @@ vfs_s_retrieve_file (struct vfs_class *me, struct vfs_s_inode *ino)
     vfs_path_t *tmp_vpath;
     struct vfs_s_subclass *s = VFS_SUBCLASS (me);
 
-    if ((me->flags & VFS_USETMP) == 0)
+    if ((me->flags & VFSF_USETMP) == 0)
         return (-1);
 
     handle = vfs_mkstemps (&tmp_vpath, me->name, ino->ent->name);
-    ino->localname = g_strdup (vfs_path_as_str (tmp_vpath));
-    vfs_path_free (tmp_vpath);
+    ino->localname = vfs_path_free (tmp_vpath, FALSE);
     if (handle == -1)
     {
         me->verrno = errno;
@@ -1510,7 +1519,7 @@ vfs_init_class (struct vfs_class *vclass, const char *name, vfs_flags_t flags, c
     vclass->open = vfs_s_open;
     vclass->close = vfs_s_close;
     vclass->read = vfs_s_read;
-    if ((vclass->flags & VFS_READONLY) == 0)
+    if ((vclass->flags & VFSF_READONLY) == 0)
         vclass->write = vfs_s_write;
     vclass->opendir = vfs_s_opendir;
     vclass->readdir = vfs_s_readdir;
@@ -1526,7 +1535,7 @@ vfs_init_class (struct vfs_class *vclass, const char *name, vfs_flags_t flags, c
     vclass->nothingisopen = vfs_s_nothingisopen;
     vclass->free = vfs_s_free;
     vclass->setctl = vfs_s_setctl;
-    if ((vclass->flags & VFS_USETMP) != 0)
+    if ((vclass->flags & VFSF_USETMP) != 0)
     {
         vclass->getlocalcopy = vfs_s_getlocalcopy;
         vclass->ungetlocalcopy = vfs_s_ungetlocalcopy;
@@ -1549,9 +1558,9 @@ vfs_init_subclass (struct vfs_s_subclass *sub, const char *name, vfs_flags_t fla
     start = (char *) sub + sizeof (struct vfs_class);
     memset (start, 0, len);
 
-    if ((vclass->flags & VFS_USETMP) != 0)
+    if ((vclass->flags & VFSF_USETMP) != 0)
         sub->find_entry = vfs_s_find_entry_linear;
-    else if ((vclass->flags & VFS_REMOTE) != 0)
+    else if ((vclass->flags & VFSF_REMOTE) != 0)
         sub->find_entry = vfs_s_find_entry_linear;
     else
         sub->find_entry = vfs_s_find_entry_tree;
@@ -1564,13 +1573,13 @@ vfs_init_subclass (struct vfs_s_subclass *sub, const char *name, vfs_flags_t fla
 vfsid
 vfs_getid (const vfs_path_t * vpath)
 {
-    const vfs_path_element_t *path_element;
+    const struct vfs_class *me;
 
-    path_element = vfs_path_get_by_index (vpath, -1);
-    if (!vfs_path_element_valid (path_element) || path_element->class->getid == NULL)
+    me = vfs_path_get_last_path_vfs (vpath);
+    if (me == NULL || me->getid == NULL)
         return NULL;
 
-    return (*path_element->class->getid) (vpath);
+    return me->getid (vpath);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1708,21 +1717,23 @@ vfs_s_normalize_filename_leading_spaces (struct vfs_s_inode *root_inode, size_t
 {
     GList *iter;
 
-    for (iter = root_inode->subdir; iter != NULL; iter = g_list_next (iter))
+    for (iter = g_queue_peek_head_link (root_inode->subdir); iter != NULL;
+         iter = g_list_next (iter))
     {
         struct vfs_s_entry *entry = VFS_ENTRY (iter->data);
 
-        if ((size_t) entry->ino->data_offset > final_num_spaces)
+        if ((size_t) entry->leading_spaces > final_num_spaces)
         {
-            char *source_name = entry->name;
-            char *spacer;
+            char *source_name, *spacer;
 
-            spacer = g_strnfill (entry->ino->data_offset - final_num_spaces, ' ');
-            entry->name = g_strdup_printf ("%s%s", spacer, source_name);
+            source_name = entry->name;
+            spacer = g_strnfill ((size_t) entry->leading_spaces - final_num_spaces, ' ');
+            entry->name = g_strconcat (spacer, source_name, (char *) NULL);
             g_free (spacer);
             g_free (source_name);
         }
-        entry->ino->data_offset = -1;
+
+        entry->leading_spaces = -1;
     }
 }
 
