@@ -1,12 +1,12 @@
 /*
    Virtual File System path handlers
 
-   Copyright (C) 2011-2019
+   Copyright (C) 2011-2024
    Free Software Foundation, Inc.
 
    Written by:
    Slava Zanko <slavazanko@gmail.com>, 2011, 2013
-   Andrew Borodin <aborodin@vmail.ru>, 2013
+   Andrew Borodin <aborodin@vmail.ru>, 2013-2022
 
    This file is part of the Midnight Commander.
 
@@ -34,6 +34,8 @@
 
 #include <config.h>
 
+#include <errno.h>
+
 #include "lib/global.h"
 #include "lib/strutil.h"
 #include "lib/util.h"           /* mc_build_filename() */
@@ -52,8 +54,11 @@ extern GPtrArray *vfs__classes_list;
 
 /*** file scope type declarations ****************************************************************/
 
+/*** forward declarations (file scope functions) *************************************************/
+
 /*** file scope variables ************************************************************************/
 
+/* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
@@ -375,7 +380,7 @@ vfs_path_from_str_deprecated_parser (char *path)
     struct vfs_class *class;
     const char *local, *op;
 
-    vpath = vfs_path_new ();
+    vpath = vfs_path_new (FALSE);
 
     while ((class = _vfs_split_with_semi_skip_count (path, &local, &op, 0)) != NULL)
     {
@@ -438,8 +443,7 @@ vfs_path_from_str_uri_parser (char *path)
     vfs_path_element_t *element;
     char *url_delimiter;
 
-    vpath = vfs_path_new ();
-    vpath->relative = path != NULL && !IS_PATH_SEP (*path);
+    vpath = vfs_path_new (path != NULL && !IS_PATH_SEP (*path));
 
     while ((url_delimiter = g_strrstr (path, VFS_PATH_URL_DELIMITER)) != NULL)
     {
@@ -461,7 +465,7 @@ vfs_path_from_str_uri_parser (char *path)
 
         url_delimiter += strlen (VFS_PATH_URL_DELIMITER);
 
-        if (element->class != NULL && (element->class->flags & VFS_REMOTE) != 0)
+        if (element->class != NULL && (element->class->flags & VFSF_REMOTE) != 0)
         {
             char *slash_pointer;
 
@@ -530,10 +534,10 @@ static void
 vfs_path_tokens_add_class_info (const vfs_path_element_t * element, GString * ret_tokens,
                                 GString * element_tokens)
 {
-    if (((element->class->flags & VFS_LOCAL) == 0 || ret_tokens->len > 0)
+    if (((element->class->flags & VFSF_LOCAL) == 0 || ret_tokens->len > 0)
         && element_tokens->len > 0)
     {
-        char *url_str;
+        GString *url_str;
 
         if (ret_tokens->len > 0 && !IS_PATH_SEP (ret_tokens->str[ret_tokens->len - 1]))
             g_string_append_c (ret_tokens, PATH_SEP);
@@ -542,13 +546,12 @@ vfs_path_tokens_add_class_info (const vfs_path_element_t * element, GString * re
         g_string_append (ret_tokens, VFS_PATH_URL_DELIMITER);
 
         url_str = vfs_path_build_url_params_str (element, TRUE);
-        if (*url_str != '\0')
+        if (url_str != NULL)
         {
-            g_string_append (ret_tokens, url_str);
+            g_string_append_len (ret_tokens, url_str->str, url_str->len);
             g_string_append_c (ret_tokens, PATH_SEP);
+            g_string_free (url_str, TRUE);
         }
-
-        g_free (url_str);
     }
 
 #ifdef HAVE_CHARSET
@@ -595,7 +598,8 @@ vfs_path_strip_home (const char *dir)
 
 #define vfs_append_from_path(appendfrom, is_relative) \
 { \
-    if ((flags & VPF_STRIP_HOME) && element_index == 0 && (element->class->flags & VFS_LOCAL) != 0) \
+    if ((flags & VPF_STRIP_HOME) && element_index == 0 && \
+        (element->class->flags & VFSF_LOCAL) != 0) \
     { \
         char *stripped_home_str; \
         stripped_home_str = vfs_path_strip_home (appendfrom); \
@@ -626,7 +630,9 @@ vfs_path_to_str_flags (const vfs_path_t * vpath, int elements_count, vfs_path_fl
 {
     int element_index;
     GString *buffer;
-    GString *recode_buffer;
+#ifdef HAVE_CHARSET
+    GString *recode_buffer = NULL;
+#endif
 
     if (vpath == NULL)
         return NULL;
@@ -638,7 +644,6 @@ vfs_path_to_str_flags (const vfs_path_t * vpath, int elements_count, vfs_path_fl
         elements_count = vfs_path_elements_count (vpath) + elements_count;
 
     buffer = g_string_new ("");
-    recode_buffer = g_string_new ("");
 
     for (element_index = 0; element_index < elements_count; element_index++)
     {
@@ -648,7 +653,8 @@ vfs_path_to_str_flags (const vfs_path_t * vpath, int elements_count, vfs_path_fl
         element = vfs_path_get_by_index (vpath, element_index);
         if (element->vfs_prefix != NULL)
         {
-            char *url_str;
+            GString *url_str;
+
             if (!is_relative && (buffer->len == 0 || !IS_PATH_SEP (buffer->str[buffer->len - 1])))
                 g_string_append_c (buffer, PATH_SEP);
 
@@ -656,14 +662,12 @@ vfs_path_to_str_flags (const vfs_path_t * vpath, int elements_count, vfs_path_fl
             g_string_append (buffer, VFS_PATH_URL_DELIMITER);
 
             url_str = vfs_path_build_url_params_str (element, !(flags & VPF_STRIP_PASSWORD));
-
-            if (*url_str != '\0')
+            if (url_str != NULL)
             {
-                g_string_append (buffer, url_str);
+                g_string_append_len (buffer, url_str->str, url_str->len);
                 g_string_append_c (buffer, PATH_SEP);
+                g_string_free (url_str, TRUE);
             }
-
-            g_free (url_str);
         }
 
 #ifdef HAVE_CHARSET
@@ -677,9 +681,14 @@ vfs_path_to_str_flags (const vfs_path_t * vpath, int elements_count, vfs_path_fl
                 g_string_append (buffer, VFS_ENCODING_PREFIX);
                 g_string_append (buffer, element->encoding);
             }
+
+            if (recode_buffer == NULL)
+                recode_buffer = g_string_sized_new (32);
+            else
+                g_string_set_size (recode_buffer, 0);
+
             str_vfs_convert_from (element->dir.converter, element->path, recode_buffer);
             vfs_append_from_path (recode_buffer->str, is_relative);
-            g_string_set_size (recode_buffer, 0);
         }
         else
 #endif
@@ -687,7 +696,12 @@ vfs_path_to_str_flags (const vfs_path_t * vpath, int elements_count, vfs_path_fl
             vfs_append_from_path (element->path, is_relative);
         }
     }
-    g_string_free (recode_buffer, TRUE);
+
+#ifdef HAVE_CHARSET
+    if (recode_buffer != NULL)
+        g_string_free (recode_buffer, TRUE);
+#endif
+
     return g_string_free (buffer, FALSE);
 }
 
@@ -770,12 +784,13 @@ vfs_path_from_str (const char *path_str)
  */
 
 vfs_path_t *
-vfs_path_new (void)
+vfs_path_new (gboolean relative)
 {
     vfs_path_t *vpath;
 
     vpath = g_new0 (vfs_path_t, 1);
     vpath->path = g_array_new (FALSE, TRUE, sizeof (vfs_path_element_t *));
+    vpath->relative = relative;
 
     return vpath;
 }
@@ -814,23 +829,36 @@ vfs_path_add_element (vfs_path_t * vpath, const vfs_path_element_t * path_elemen
 /*
  * Get one path element by index.
  *
- * @param vpath pointer to vfs_path_t object
- * @param element_index element index. May have negative value (in this case count was started at the end of list).
+ * @param vpath pointer to vfs_path_t object.
+ *              May be NULL. In this case NULL is returned and errno set to 0.
+ * @param element_index element index. May have negative value (in this case count was started at
+ *                      the end of list). If @element_index is out of range, NULL is returned and
+ *                      errno set to EINVAL.
  *
- * @return path element.
+ * @return path element
  */
 
 const vfs_path_element_t *
 vfs_path_get_by_index (const vfs_path_t * vpath, int element_index)
 {
+    int n;
+
     if (vpath == NULL)
+    {
+        errno = 0;
         return NULL;
+    }
+
+    n = vfs_path_elements_count (vpath);
 
     if (element_index < 0)
-        element_index += vfs_path_elements_count (vpath);
+        element_index += n;
 
-    if (element_index < 0)
-        vfs_die ("vfs_path_get_by_index: incorrect index!");
+    if (element_index < 0 || element_index > n)
+    {
+        errno = EINVAL;
+        return NULL;
+    }
 
     return g_array_index (vpath->path, vfs_path_element_t *, element_index);
 }
@@ -917,8 +945,7 @@ vfs_path_clone (const vfs_path_t * vpath)
     if (vpath == NULL)
         return NULL;
 
-    new_vpath = vfs_path_new ();
-    new_vpath->relative = vpath->relative;
+    new_vpath = vfs_path_new (vpath->relative);
 
     for (vpath_element_index = 0; vpath_element_index < vfs_path_elements_count (vpath);
          vpath_element_index++)
@@ -938,16 +965,19 @@ vfs_path_clone (const vfs_path_t * vpath)
  * Free vfs_path_t object.
  *
  * @param vpath pointer to vfs_path_t object
+ * @param free_str if TRUE the string representation of vpath is freed as well
  *
+ * @return the string representation of vpath (i.e. NULL if free_str is TRUE)
  */
 
-void
-vfs_path_free (vfs_path_t * vpath)
+char *
+vfs_path_free (vfs_path_t * vpath, gboolean free_str)
 {
     int vpath_element_index;
+    char *ret;
 
     if (vpath == NULL)
-        return;
+        return NULL;
 
     for (vpath_element_index = 0; vpath_element_index < vfs_path_elements_count (vpath);
          vpath_element_index++)
@@ -959,8 +989,18 @@ vfs_path_free (vfs_path_t * vpath)
     }
 
     g_array_free (vpath->path, TRUE);
-    g_free (vpath->str);
+
+    if (!free_str)
+        ret = vpath->str;
+    else
+    {
+        g_free (vpath->str);
+        ret = NULL;
+    }
+
     g_free (vpath);
+
+    return ret;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1043,7 +1083,7 @@ vfs_path_element_need_cleanup_converter (const vfs_path_element_t * element)
  * @param vpath pointer to path structure
  * encoding name of charset
  *
- * @return pointer to path structure (for use function in anoter functions)
+ * @return pointer to path structure (for use function in another functions)
  */
 vfs_path_t *
 vfs_path_change_encoding (vfs_path_t * vpath, const char *encoding)
@@ -1149,7 +1189,7 @@ vfs_path_deserialize (const char *data, GError ** mcerror)
     if (cpath == NULL)
         return NULL;
 
-    vpath = vfs_path_new ();
+    vpath = vfs_path_new (FALSE);
 
     for (element_index = 0;; element_index++)
     {
@@ -1166,7 +1206,7 @@ vfs_path_deserialize (const char *data, GError ** mcerror)
         eclass = vfs_get_class_by_name (cfg_value);
         if (eclass == NULL)
         {
-            vfs_path_free (vpath);
+            vfs_path_free (vpath, TRUE);
             g_set_error (mcerror, MC_ERROR, 0, "Unable to find VFS class by name '%s'", cfg_value);
             g_free (cfg_value);
             mc_config_deinit (cpath);
@@ -1197,7 +1237,7 @@ vfs_path_deserialize (const char *data, GError ** mcerror)
     mc_config_deinit (cpath);
     if (vfs_path_elements_count (vpath) == 0)
     {
-        vfs_path_free (vpath);
+        vfs_path_free (vpath, TRUE);
         g_set_error (mcerror, MC_ERROR, 0, "No any path elements found");
         return NULL;
     }
@@ -1289,7 +1329,7 @@ vfs_path_append_vpath_new (const vfs_path_t * first_vpath, ...)
     if (first_vpath == NULL)
         return NULL;
 
-    ret_vpath = vfs_path_new ();
+    ret_vpath = vfs_path_new (FALSE);
 
     va_start (args, first_vpath);
     do
@@ -1472,10 +1512,10 @@ vfs_path_vtokens_get (const vfs_path_t * vpath, ssize_t start_position, ssize_t
  * @param element path element
  * @param keep_password TRUE or FALSE
  *
- * @return newly allocated string
+ * @return newly allocated non-empty string or NULL
  */
 
-char *
+GString *
 vfs_path_build_url_params_str (const vfs_path_element_t * element, gboolean keep_password)
 {
     GString *buffer;
@@ -1483,7 +1523,7 @@ vfs_path_build_url_params_str (const vfs_path_element_t * element, gboolean keep
     if (element == NULL)
         return NULL;
 
-    buffer = g_string_new ("");
+    buffer = g_string_sized_new (64);
 
     if (element->user != NULL)
         g_string_append (buffer, element->user);
@@ -1511,7 +1551,11 @@ vfs_path_build_url_params_str (const vfs_path_element_t * element, gboolean keep
         g_string_append_printf (buffer, "%d", element->port);
     }
 
-    return g_string_free (buffer, FALSE);
+    if (buffer->len != 0)
+        return buffer;
+
+    g_string_free (buffer, TRUE);
+    return NULL;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1523,24 +1567,25 @@ vfs_path_build_url_params_str (const vfs_path_element_t * element, gboolean keep
  * @return newly allocated string
  */
 
-char *
+GString *
 vfs_path_element_build_pretty_path_str (const vfs_path_element_t * element)
 {
-    char *url_params;
-    GString *pretty_path;
+    GString *url_params, *pretty_path;
 
     pretty_path = g_string_new (element->class->prefix);
     g_string_append (pretty_path, VFS_PATH_URL_DELIMITER);
 
     url_params = vfs_path_build_url_params_str (element, FALSE);
-    g_string_append (pretty_path, url_params);
-    g_free (url_params);
+    if (url_params != NULL)
+    {
+        g_string_append_len (pretty_path, url_params->str, url_params->len);
+        g_string_free (url_params, TRUE);
+    }
 
     if (!IS_PATH_SEP (*element->path))
         g_string_append_c (pretty_path, PATH_SEP);
 
-    g_string_append (pretty_path, element->path);
-    return g_string_free (pretty_path, FALSE);
+    return g_string_append (pretty_path, element->path);
 }
 
 /* --------------------------------------------------------------------------------------------- */
