@@ -1,7 +1,7 @@
 /*
    Widgets for the Midnight Commander
 
-   Copyright (C) 1994-2021
+   Copyright (C) 1994-2024
    Free Software Foundation, Inc.
 
    Authors:
@@ -10,7 +10,7 @@
    Jakub Jelinek, 1995
    Andrej Borsenkow, 1996
    Norbert Warmuth, 1997
-   Andrew Borodin <aborodin@vmail.ru>, 2009, 2010, 2013, 2016
+   Andrew Borodin <aborodin@vmail.ru>, 2009-2022
 
    This file is part of the Midnight Commander.
 
@@ -55,9 +55,13 @@ const global_keymap_t *listbox_map = NULL;
 
 /*** file scope type declarations ****************************************************************/
 
+/*** forward declarations (file scope functions) *************************************************/
+
 /*** file scope variables ************************************************************************/
 
+/* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
+/* --------------------------------------------------------------------------------------------- */
 
 static int
 listbox_entry_cmp (const void *a, const void *b, void *user_data)
@@ -86,16 +90,16 @@ listbox_entry_free (void *data)
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-listbox_drawscroll (WListbox * l)
+listbox_drawscroll (const WListbox * l)
 {
-    Widget *w = WIDGET (l);
+    const WRect *w = &CONST_WIDGET (l)->rect;
     int max_line = w->lines - 1;
     int line = 0;
     int i;
     int length;
 
     /* Are we at the top? */
-    widget_gotoyx (w, 0, w->cols);
+    widget_gotoyx (l, 0, w->cols);
     if (l->top == 0)
         tty_print_one_vline (TRUE);
     else
@@ -112,11 +116,11 @@ listbox_drawscroll (WListbox * l)
 
     /* Now draw the nice relative pointer */
     if (!g_queue_is_empty (l->list))
-        line = 1 + ((l->pos * (w->lines - 2)) / length);
+        line = 1 + ((l->current * (w->lines - 2)) / length);
 
     for (i = 1; i < max_line; i++)
     {
-        widget_gotoyx (w, i, w->cols);
+        widget_gotoyx (l, i, w->cols);
         if (i != line)
             tty_print_one_vline (TRUE);
         else
@@ -129,7 +133,8 @@ listbox_drawscroll (WListbox * l)
 static void
 listbox_draw (WListbox * l, gboolean focused)
 {
-    Widget *w = WIDGET (l);
+    Widget *wl = WIDGET (l);
+    const WRect *w = &CONST_WIDGET (l)->rect;
     const int *colors;
     gboolean disabled;
     int normalc, selc;
@@ -139,9 +144,9 @@ listbox_draw (WListbox * l, gboolean focused)
     int i;
     int sel_line = -1;
 
-    colors = widget_get_colors (w);
+    colors = widget_get_colors (wl);
 
-    disabled = widget_get_state (w, WST_DISABLED);
+    disabled = widget_get_state (wl, WST_DISABLED);
     normalc = disabled ? DISABLED_COLOR : colors[DLG_COLOR_NORMAL];
     selc = disabled ? DISABLED_COLOR : colors[focused ? DLG_COLOR_HOT_FOCUS : DLG_COLOR_FOCUS];
 
@@ -159,7 +164,7 @@ listbox_draw (WListbox * l, gboolean focused)
         const char *text = "";
 
         /* Display the entry */
-        if (pos == l->pos && sel_line == -1)
+        if (pos == l->current && sel_line == -1)
         {
             sel_line = i;
             tty_setcolor (selc);
@@ -228,8 +233,8 @@ listbox_fwd (WListbox * l, gboolean wrap)
 {
     if (!listbox_is_empty (l))
     {
-        if ((guint) l->pos + 1 < g_queue_get_length (l->list))
-            listbox_select_entry (l, l->pos + 1);
+        if ((guint) l->current + 1 < g_queue_get_length (l->list))
+            listbox_set_current (l, l->current + 1);
         else if (wrap)
             listbox_select_first (l);
     }
@@ -240,7 +245,7 @@ listbox_fwd (WListbox * l, gboolean wrap)
 static void
 listbox_fwd_n (WListbox * l, int n)
 {
-    listbox_select_entry (l, MIN (l->pos + n, LISTBOX_LAST (l)));
+    listbox_set_current (l, MIN (l->current + n, LISTBOX_LAST (l)));
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -250,8 +255,8 @@ listbox_back (WListbox * l, gboolean wrap)
 {
     if (!listbox_is_empty (l))
     {
-        if (l->pos > 0)
-            listbox_select_entry (l, l->pos - 1);
+        if (l->current > 0)
+            listbox_set_current (l, l->current - 1);
         else if (wrap)
             listbox_select_last (l);
     }
@@ -262,7 +267,7 @@ listbox_back (WListbox * l, gboolean wrap)
 static void
 listbox_back_n (WListbox * l, int n)
 {
-    listbox_select_entry (l, MAX (l->pos - n, 0));
+    listbox_set_current (l, MAX (l->current - n, 0));
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -271,7 +276,7 @@ static cb_ret_t
 listbox_execute_cmd (WListbox * l, long command)
 {
     cb_ret_t ret = MSG_HANDLED;
-    Widget *w = WIDGET (l);
+    const WRect *w = &CONST_WIDGET (l)->rect;
 
     if (l->list == NULL || g_queue_is_empty (l->list))
         return MSG_NOT_HANDLED;
@@ -304,7 +309,7 @@ listbox_execute_cmd (WListbox * l, long command)
 
             length = g_queue_get_length (l->list);
 
-            is_last = (l->pos + 1 >= length);
+            is_last = (l->current + 1 >= length);
             is_more = (l->top + w->lines >= length);
 
             listbox_remove_current (l);
@@ -346,7 +351,7 @@ listbox_key (WListbox * l, int key)
     /* focus on listbox item N by '0'..'9' keys */
     if (key >= '0' && key <= '9')
     {
-        listbox_select_entry (l, key - '0');
+        listbox_set_current (l, key - '0');
         return MSG_HANDLED;
     }
 
@@ -360,7 +365,7 @@ listbox_key (WListbox * l, int key)
 
 /* Listbox item adding function */
 static inline void
-listbox_append_item (WListbox * l, WLEntry * e, listbox_append_t pos)
+listbox_add_entry (WListbox * l, WLEntry * e, listbox_append_t pos)
 {
     if (l->list == NULL)
     {
@@ -375,11 +380,11 @@ listbox_append_item (WListbox * l, WLEntry * e, listbox_append_t pos)
         break;
 
     case LISTBOX_APPEND_BEFORE:
-        g_queue_insert_before (l->list, g_queue_peek_nth_link (l->list, (guint) l->pos), e);
+        g_queue_insert_before (l->list, g_queue_peek_nth_link (l->list, (guint) l->current), e);
         break;
 
     case LISTBOX_APPEND_AFTER:
-        g_queue_insert_after (l->list, g_queue_peek_nth_link (l->list, (guint) l->pos), e);
+        g_queue_insert_after (l->list, g_queue_peek_nth_link (l->list, (guint) l->current), e);
         break;
 
     case LISTBOX_APPEND_SORTED:
@@ -421,7 +426,7 @@ listbox_do_action (WListbox * l)
         WDialog *h = DIALOG (WIDGET (l)->owner);
 
         h->ret_value = B_ENTER;
-        dlg_stop (h);
+        dlg_close (h);
     }
 }
 
@@ -430,7 +435,7 @@ listbox_do_action (WListbox * l)
 static void
 listbox_run_hotkey (WListbox * l, int pos)
 {
-    listbox_select_entry (l, pos);
+    listbox_set_current (l, pos);
     listbox_on_change (l);
     listbox_do_action (l);
 }
@@ -501,15 +506,15 @@ static void
 listbox_mouse_callback (Widget * w, mouse_msg_t msg, mouse_event_t * event)
 {
     WListbox *l = LISTBOX (w);
-    int old_pos;
+    int old_current;
 
-    old_pos = l->pos;
+    old_current = l->current;
 
     switch (msg)
     {
     case MSG_MOUSE_DOWN:
         widget_select (w);
-        listbox_select_entry (l, listbox_y_pos (l, event->y));
+        listbox_set_current (l, listbox_y_pos (l, event->y));
         break;
 
     case MSG_MOUSE_SCROLL_UP:
@@ -522,11 +527,11 @@ listbox_mouse_callback (Widget * w, mouse_msg_t msg, mouse_event_t * event)
 
     case MSG_MOUSE_DRAG:
         event->result.repeat = TRUE;    /* It'd be functional even without this. */
-        listbox_select_entry (l, listbox_y_pos (l, event->y));
+        listbox_set_current (l, listbox_y_pos (l, event->y));
         break;
 
     case MSG_MOUSE_CLICK:
-        /* We don't call listbox_select_entry() here: MSG_MOUSE_DOWN/DRAG did this already. */
+        /* We don't call listbox_set_current() here: MSG_MOUSE_DOWN/DRAG did this already. */
         if (event->count == GPM_DOUBLE) /* Double click */
             listbox_do_action (l);
         break;
@@ -536,7 +541,7 @@ listbox_mouse_callback (Widget * w, mouse_msg_t msg, mouse_event_t * event)
     }
 
     /* If the selection has changed, we redraw the widget and notify the dialog. */
-    if (l->pos != old_pos)
+    if (l->current != old_current)
         listbox_on_change (l);
 }
 
@@ -547,20 +552,19 @@ listbox_mouse_callback (Widget * w, mouse_msg_t msg, mouse_event_t * event)
 WListbox *
 listbox_new (int y, int x, int height, int width, gboolean deletable, lcback_fn callback)
 {
+    WRect r = { y, x, 1, width };
     WListbox *l;
     Widget *w;
 
-    if (height <= 0)
-        height = 1;
-
     l = g_new (WListbox, 1);
     w = WIDGET (l);
-    widget_init (w, y, x, height, width, listbox_callback, listbox_mouse_callback);
+    r.lines = height > 0 ? height : 1;
+    widget_init (w, &r, listbox_callback, listbox_mouse_callback);
     w->options |= WOP_SELECTABLE | WOP_WANT_HOTKEY;
     w->keymap = listbox_map;
 
     l->list = NULL;
-    l->top = l->pos = 0;
+    l->top = l->current = 0;
     l->deletable = deletable;
     l->callback = callback;
     l->allow_duplicates = TRUE;
@@ -621,11 +625,11 @@ listbox_search_data (WListbox * l, const void *data)
 
 /* --------------------------------------------------------------------------------------------- */
 
-/* Selects the first entry and scrolls the list to the top */
+/* Select the first entry and scrolls the list to the top */
 void
 listbox_select_first (WListbox * l)
 {
-    l->pos = l->top = 0;
+    l->current = l->top = 0;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -634,19 +638,19 @@ listbox_select_first (WListbox * l)
 void
 listbox_select_last (WListbox * l)
 {
-    int lines = WIDGET (l)->lines;
+    int lines = WIDGET (l)->rect.lines;
     int length;
 
     length = listbox_get_length (l);
 
-    l->pos = DOZ (length, 1);
+    l->current = DOZ (length, 1);
     l->top = DOZ (length, lines);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 void
-listbox_select_entry (WListbox * l, int dest)
+listbox_set_current (WListbox * l, int dest)
 {
     GList *le;
     int pos;
@@ -663,22 +667,22 @@ listbox_select_entry (WListbox * l, int dest)
 
         if (pos == dest)
         {
-            l->pos = dest;
+            l->current = dest;
             if (!top_seen)
-                l->top = l->pos;
+                l->top = l->current;
             else
             {
-                int lines = WIDGET (l)->lines;
+                int lines = WIDGET (l)->rect.lines;
 
-                if (l->pos - l->top >= lines)
-                    l->top = l->pos - lines + 1;
+                if (l->current - l->top >= lines)
+                    l->top = l->current - lines + 1;
             }
             return;
         }
     }
 
     /* If we are unable to find it, set decent values */
-    l->pos = l->top = 0;
+    l->current = l->top = 0;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -699,7 +703,7 @@ listbox_get_current (WListbox * l, char **string, void **extra)
     gboolean ok;
 
     if (l != NULL)
-        e = listbox_get_nth_item (l, l->pos);
+        e = listbox_get_nth_entry (l, l->current);
 
     ok = (e != NULL);
 
@@ -713,7 +717,7 @@ listbox_get_current (WListbox * l, char **string, void **extra)
 /* --------------------------------------------------------------------------------------------- */
 
 WLEntry *
-listbox_get_nth_item (const WListbox * l, int pos)
+listbox_get_nth_entry (const WListbox * l, int pos)
 {
     if (!listbox_is_empty (l) && pos >= 0)
     {
@@ -745,16 +749,16 @@ listbox_remove_current (WListbox * l)
         GList *current;
         int length;
 
-        current = g_queue_peek_nth_link (l->list, (guint) l->pos);
+        current = g_queue_peek_nth_link (l->list, (guint) l->current);
         listbox_entry_free (current->data);
         g_queue_delete_link (l->list, current);
 
         length = g_queue_get_length (l->list);
 
         if (length == 0)
-            l->top = l->pos = 0;
-        else if (l->pos >= length)
-            l->pos = length - 1;
+            l->top = l->current = 0;
+        else if (l->current >= length)
+            l->current = length - 1;
     }
 }
 
@@ -796,7 +800,7 @@ listbox_remove_list (WListbox * l)
             l->list = NULL;
         }
 
-        l->pos = l->top = 0;
+        l->current = l->top = 0;
     }
 }
 
@@ -820,7 +824,7 @@ listbox_add_item (WListbox * l, listbox_append_t pos, int hotkey, const char *te
     entry->free_data = free_data;
     entry->hotkey = hotkey;
 
-    listbox_append_item (l, entry, pos);
+    listbox_add_entry (l, entry, pos);
 
     return entry->text;
 }
