@@ -1,7 +1,7 @@
 /*
    Handle command line arguments.
 
-   Copyright (C) 2009-2018
+   Copyright (C) 2009-2024
    Free Software Foundation, Inc.
 
    Written by:
@@ -34,10 +34,6 @@
 #include "lib/vfs/vfs.h"
 #include "lib/util.h"           /* x_basename() */
 
-#ifdef ENABLE_VFS_SMB
-#include "src/vfs/smbfs/smbfs.h"        /* smbfs_set_debugf()  */
-#endif
-
 #include "src/textconf.h"
 
 #include "src/args.h"
@@ -59,17 +55,12 @@ gboolean mc_args__nokeymap = FALSE;
 
 char *mc_args__last_wd_file = NULL;
 
-/* when enabled NETCODE, use folowing file as logfile */
+/* when enabled NETCODE, use following file as logfile */
 char *mc_args__netfs_logfile = NULL;
 
 /* keymap file */
 char *mc_args__keymap_file = NULL;
 
-/* Debug level */
-#ifdef ENABLE_VFS_SMB
-int mc_args__debug_level = 0;
-#endif
-
 void *mc_run_param0 = NULL;
 char *mc_run_param1 = NULL;
 
@@ -77,17 +68,18 @@ char *mc_run_param1 = NULL;
 
 /*** file scope type declarations ****************************************************************/
 
-/*** file scope variables ************************************************************************/
+/*** forward declarations (file scope functions) *************************************************/
 
-/* If true, show version info and exit */
-static gboolean mc_args__show_version = FALSE;
-
-/* forward declarations */
 static gboolean parse_mc_e_argument (const gchar * option_name, const gchar * value,
                                      gpointer data, GError ** mcerror);
 static gboolean parse_mc_v_argument (const gchar * option_name, const gchar * value,
                                      gpointer data, GError ** mcerror);
 
+/*** file scope variables ************************************************************************/
+
+/* If true, show version info and exit */
+static gboolean mc_args__show_version = FALSE;
+
 static GOptionContext *context;
 
 #ifdef ENABLE_SUBSHELL
@@ -95,7 +87,9 @@ static gboolean mc_args__nouse_subshell = FALSE;
 #endif /* ENABLE_SUBSHELL */
 static gboolean mc_args__show_datadirs = FALSE;
 static gboolean mc_args__show_datadirs_extended = FALSE;
+#ifdef ENABLE_CONFIGURE_ARGS
 static gboolean mc_args__show_configure_opts = FALSE;
+#endif
 
 static GOptionGroup *main_group;
 
@@ -125,6 +119,7 @@ static const GOptionEntry argument_main_table[] = {
      NULL
     },
 
+#ifdef ENABLE_CONFIGURE_ARGS
     /* show configure options */
     {
      "configure-options", '\0', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE,
@@ -132,6 +127,7 @@ static const GOptionEntry argument_main_table[] = {
      N_("Print configure options"),
      NULL
     },
+#endif
 
     {
      "printwd", 'P', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING,
@@ -165,14 +161,6 @@ static const GOptionEntry argument_main_table[] = {
      N_("<file>")
     },
 #endif /* ENABLE_VFS_FTP */
-#ifdef ENABLE_VFS_SMB
-    {
-     "debuglevel", 'D', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_INT,
-     &mc_args__debug_level,
-     N_("Set debug level"),
-     N_("<integer>")
-    },
-#endif /* ENABLE_VFS_SMB */
 
     {
      /* handle arguments manually */
@@ -187,11 +175,10 @@ static const GOptionEntry argument_main_table[] = {
      "edit", 'e', G_OPTION_FLAG_IN_MAIN | G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
      (gpointer) parse_mc_e_argument,
      N_("Edit files"),
-     N_("<file> ...") },
+     N_("<file> ...")
+    },
 
-    {
-     NULL, '\0', 0, 0, NULL, NULL, NULL /* Complete struct initialization */
-    }
+    G_OPTION_ENTRY_NULL
     /* *INDENT-ON* */
 };
 
@@ -274,9 +261,7 @@ static const GOptionEntry argument_terminal_table[] = {
      NULL
     },
 
-    {
-     NULL, '\0', 0, 0, NULL, NULL, NULL /* Complete struct initialization */
-    }
+    G_OPTION_ENTRY_NULL
     /* *INDENT-ON* */
 };
 
@@ -316,9 +301,7 @@ static const GOptionEntry argument_color_table[] = {
      N_("<string>")
     },
 
-    {
-     NULL, '\0', 0, 0, NULL, NULL, NULL /* Complete struct initialization */
-    }
+    G_OPTION_ENTRY_NULL
     /* *INDENT-ON* */
 };
 
@@ -329,9 +312,10 @@ static gchar *mc_args__loc__footer_string = NULL;
 static gchar *mc_args__loc__header_string = NULL;
 static gchar *mc_args__loc__usage_string = NULL;
 
-/*** file scope functions ************************************************************************/
-
 /* --------------------------------------------------------------------------------------------- */
+/*** file scope functions ************************************************************************/
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 mc_args_clean_temp_help_strings (void)
 {
@@ -346,7 +330,7 @@ mc_args_clean_temp_help_strings (void)
 static GOptionGroup *
 mc_args_new_color_group (void)
 {
-/* *INDENT-OFF* */
+    /* *INDENT-OFF* */
     /* FIXME: to preserve translations, lines should be split. */
     mc_args__loc__colors_string = g_strdup_printf ("%s\n%s",
                                                    /* TRANSLATORS: don't translate keywords */
@@ -361,7 +345,7 @@ mc_args_new_color_group (void)
                                                      "                 errdhotfocus\n"
                                                      "   Menus:        menunormal, menuhot, menusel, menuhotsel, menuinactive\n"
                                                      "   Popup menus:  pmenunormal, pmenusel, pmenutitle\n"
-                                                     "   Editor:       editnormal, editbold, editmarked, editwhitespace,\n"
+                                                     "   Editor:       editnormal, editbold, editmarked, editwhitespace, editnonprintable,\n"
                                                      "                 editlinestate, editbg, editframe, editframeactive\n"
                                                      "                 editframedrag\n"
                                                      "   Viewer:       viewnormal,viewbold, viewunderline, viewselected\n"
@@ -376,7 +360,7 @@ mc_args_new_color_group (void)
                                                     "Attributes:\n"
                                                     "   bold, italic, underline, reverse, blink; append more with '+'\n")
                                                     );
-/* *INDENT-ON* */
+    /* *INDENT-ON* */
 
     return g_option_group_new ("color", mc_args__loc__colors_string,
                                _("Color options"), NULL, NULL);
@@ -423,7 +407,8 @@ mc_args_add_extended_info_to_help (void)
                                                    ("\n"
                                                     "Please send any bug reports (including the output of 'mc -V')\n"
                                                     "as tickets at www.midnight-commander.org\n"));
-    mc_args__loc__header_string = g_strdup_printf (_("GNU Midnight Commander %s\n"), VERSION);
+    mc_args__loc__header_string =
+        g_strdup_printf (_("GNU Midnight Commander %s\n"), mc_global.mc_version);
 
     g_option_context_set_description (context, mc_args__loc__footer_string);
     g_option_context_set_summary (context, mc_args__loc__header_string);
@@ -431,7 +416,7 @@ mc_args_add_extended_info_to_help (void)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static gchar *
+static GString *
 mc_args__convert_help_to_syscharset (const gchar * charset, const gchar * error_message_str,
                                      const gchar * help_str)
 {
@@ -448,7 +433,7 @@ mc_args__convert_help_to_syscharset (const gchar * charset, const gchar * error_
     g_free (full_help_str);
     g_iconv_close (conv);
 
-    return g_string_free (buffer, FALSE);
+    return buffer;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -595,12 +580,12 @@ parse_mcedit_arguments (int argc, char **argv)
             if (mc_stat (tmp_vpath, &st) == -1 && mc_stat (fname_vpath, &st) != -1)
             {
                 arg = mcedit_arg_vpath_new (fname_vpath, atoi (p));
-                vfs_path_free (tmp_vpath);
+                vfs_path_free (tmp_vpath, TRUE);
             }
             else
             {
                 arg = mcedit_arg_vpath_new (tmp_vpath, 0);
-                vfs_path_free (fname_vpath);
+                vfs_path_free (fname_vpath, TRUE);
             }
 
             g_free (fname);
@@ -710,13 +695,13 @@ mc_args_parse (int *argc, char ***argv, const char *translation_domain, GError *
                                   help_str);
             else
             {
-                gchar *full_help_str;
+                GString *full_help_str;
 
                 full_help_str =
                     mc_args__convert_help_to_syscharset (_system_codepage, (*mcerror)->message,
                                                          help_str);
-                mc_replace_error (mcerror, (*mcerror)->code, "%s", full_help_str);
-                g_free (full_help_str);
+                mc_replace_error (mcerror, (*mcerror)->code, "%s", full_help_str->str);
+                g_string_free (full_help_str, TRUE);
             }
             g_free (help_str);
         }
@@ -758,11 +743,13 @@ mc_args_show_info (void)
         return FALSE;
     }
 
+#ifdef ENABLE_CONFIGURE_ARGS
     if (mc_args__show_configure_opts)
     {
         show_configure_options ();
         return FALSE;
     }
+#endif
 
     return TRUE;
 }
@@ -784,26 +771,16 @@ mc_setup_by_args (int argc, char **argv, GError ** mcerror)
         mc_global.tty.use_subshell = FALSE;
 #endif /* ENABLE_SUBSHELL */
 
-#ifdef ENABLE_VFS_SMB
-    if (mc_args__debug_level != 0)
-        smbfs_set_debug (mc_args__debug_level);
-#endif /* ENABLE_VFS_SMB */
-
+#ifdef ENABLE_VFS_FTP
     if (mc_args__netfs_logfile != NULL)
     {
         vfs_path_t *vpath;
-#ifdef ENABLE_VFS_FTP
+
         vpath = vfs_path_from_str ("ftp://");
         mc_setctl (vpath, VFS_SETCTL_LOGFILE, (void *) mc_args__netfs_logfile);
-        vfs_path_free (vpath);
-#endif /* ENABLE_VFS_FTP */
-#ifdef ENABLE_VFS_SMB
-        vpath = vfs_path_from_str ("smb://");
-        mc_setctl (vpath, VFS_SETCTL_LOGFILE, (void *) mc_args__netfs_logfile);
-        vfs_path_free (vpath);
-#endif /* ENABLE_VFS_SMB */
-        (void) vpath;
+        vfs_path_free (vpath, TRUE);
     }
+#endif /* ENABLE_VFS_FTP */
 
     tmp = (argc > 0) ? argv[1] : NULL;
 
@@ -828,7 +805,7 @@ mc_setup_by_args (int argc, char **argv, GError ** mcerror)
         if (argc < 3)
         {
             mc_propagate_error (mcerror, 0, "%s\n",
-                                _("Two files are required to envoke the diffviewer."));
+                                _("Two files are required to invoke the diffviewer."));
             return FALSE;
         }
         MC_FALLTHROUGH;
@@ -861,7 +838,7 @@ mc_setup_by_args (int argc, char **argv, GError ** mcerror)
 void
 mcedit_arg_free (mcedit_arg_t * arg)
 {
-    vfs_path_free (arg->file_vpath);
+    vfs_path_free (arg->file_vpath, TRUE);
     g_free (arg);
 }
 
