@@ -1,7 +1,7 @@
 /*
    Execution routines for GNU Midnight Commander
 
-   Copyright (C) 2003-2019
+   Copyright (C) 2003-2024
    Free Software Foundation, Inc.
 
    Written by:
@@ -45,7 +45,7 @@
 #include "lib/strutil.h"        /* str_replace_all_substrings() */
 #include "lib/widget.h"
 
-#include "filemanager/midnight.h"
+#include "filemanager/filemanager.h"
 #include "filemanager/layout.h" /* use_dash() */
 #include "consaver/cons.saver.h"
 #ifdef ENABLE_SUBSHELL
@@ -63,9 +63,7 @@ int pause_after_run = pause_on_dumb_terminals;
 
 /*** file scope type declarations ****************************************************************/
 
-/*** file scope variables ************************************************************************/
-
-/*** file scope functions ************************************************************************/
+/*** forward declarations (file scope functions) *************************************************/
 
 void do_execute (const char *shell, const char *command, int flags);
 void do_executev (const char *shell, int flags, char *const argv[]);
@@ -73,6 +71,10 @@ char *execute_get_external_cmd_opts_from_config (const char *command,
                                                  const vfs_path_t * filename_vpath,
                                                  long start_line);
 
+/*** file scope variables ************************************************************************/
+
+/* --------------------------------------------------------------------------------------------- */
+/*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
 static void
@@ -99,7 +101,7 @@ static void
 edition_pre_exec (void)
 {
     if (clear_before_exec)
-        clr_scr ();
+        tty_clear_screen ();
     else
     {
         if (!(mc_global.tty.console_flag != '\0' || mc_global.tty.xterm_flag))
@@ -131,7 +133,7 @@ edition_pre_exec (void)
 static void
 do_possible_cd (const vfs_path_t * new_dir_vpath)
 {
-    if (!do_cd (new_dir_vpath, cd_exact))
+    if (!panel_cd (current_panel, new_dir_vpath, cd_exact))
         message (D_ERROR, _("Warning"), "%s",
                  _("The Commander can't change to the directory that\n"
                    "the subshell claims you are in. Perhaps you have\n"
@@ -220,7 +222,7 @@ execute_cleanup_with_vfs_arg (const vfs_path_t * filename_vpath, vfs_path_t ** l
          */
         mc_stat (*localcopy_vpath, &st);
         mc_ungetlocalcopy (filename_vpath, *localcopy_vpath, *mtime != st.st_mtime);
-        vfs_path_free (*localcopy_vpath);
+        vfs_path_free (*localcopy_vpath, TRUE);
         *localcopy_vpath = NULL;
     }
 }
@@ -240,7 +242,7 @@ execute_get_opts_from_cfg (const char *command, const char *default_str)
     {
         mc_config_t *cfg;
 
-        cfg = mc_config_init (global_profile_name, TRUE);
+        cfg = mc_config_init (mc_global.profile_name, TRUE);
         if (cfg == NULL)
             return g_strdup (default_str);
 
@@ -356,7 +358,7 @@ do_executev (const char *shell, int flags, char *const argv[])
     if (new_dir_vpath != NULL)
     {
         do_possible_cd (new_dir_vpath);
-        vfs_path_free (new_dir_vpath);
+        vfs_path_free (new_dir_vpath, TRUE);
     }
 
 #endif /* ENABLE_SUBSHELL */
@@ -364,7 +366,7 @@ do_executev (const char *shell, int flags, char *const argv[])
     if (old_vfs_dir_vpath != NULL)
     {
         mc_chdir (old_vfs_dir_vpath);
-        vfs_path_free (old_vfs_dir_vpath);
+        vfs_path_free (old_vfs_dir_vpath, TRUE);
     }
 
     if (mc_global.mc_run_mode == MC_RUN_FULL)
@@ -446,19 +448,31 @@ shell_execute (const char *command, int flags)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-toggle_panels (void)
+toggle_subshell (void)
 {
+    static gboolean message_flag = TRUE;
+
 #ifdef ENABLE_SUBSHELL
     vfs_path_t *new_dir_vpath = NULL;
 #endif /* ENABLE_SUBSHELL */
 
     SIG_ATOMIC_VOLATILE_T was_sigwinch = 0;
 
+    if (!(mc_global.tty.xterm_flag || mc_global.tty.console_flag != '\0'
+          || mc_global.tty.use_subshell || output_starts_shell))
+    {
+        if (message_flag)
+            message (D_ERROR, MSG_ERROR,
+                     _("Not an xterm or Linux console;\nthe subshell cannot be toggled."));
+        message_flag = FALSE;
+        return;
+    }
+
     channels_down ();
     disable_mouse ();
     disable_bracketed_paste ();
     if (clear_before_exec)
-        clr_scr ();
+        tty_clear_screen ();
     if (mc_global.tty.alternate_plus_minus)
         numeric_keypad_mode ();
 #ifndef HAVE_SLANG
@@ -531,22 +545,22 @@ toggle_panels (void)
      * Save sigwinch flag that will be reset in mc_refresh() called via update_panels().
      * There is some problem with screen redraw in ncurses-based mc in this situation.
      */
-    was_sigwinch = mc_global.tty.winch_flag;
-    mc_global.tty.winch_flag = 0;
+    was_sigwinch = tty_got_winch ();
+    tty_flush_winch ();
 
 #ifdef ENABLE_SUBSHELL
     if (mc_global.tty.use_subshell)
     {
-        do_load_prompt ();
-        if (new_dir_vpath != NULL)
-            do_possible_cd (new_dir_vpath);
-        if (mc_global.tty.console_flag != '\0' && output_lines)
-            show_console_contents (output_start_y,
-                                   LINES - mc_global.keybar_visible - output_lines -
-                                   1, LINES - mc_global.keybar_visible - 1);
+        if (mc_global.mc_run_mode == MC_RUN_FULL)
+        {
+            if (new_dir_vpath != NULL)
+                do_possible_cd (new_dir_vpath);
+        }
+        else if (new_dir_vpath != NULL && mc_chdir (new_dir_vpath) != -1)
+            vfs_setup_cwd ();
     }
 
-    vfs_path_free (new_dir_vpath);
+    vfs_path_free (new_dir_vpath, TRUE);
 #endif /* ENABLE_SUBSHELL */
 
     if (mc_global.mc_run_mode == MC_RUN_FULL)
@@ -555,7 +569,7 @@ toggle_panels (void)
         update_xterm_title_path ();
     }
 
-    if (was_sigwinch != 0 || mc_global.tty.winch_flag != 0)
+    if (was_sigwinch != 0 || tty_got_winch ())
         dialog_change_screen_size ();
     else
         repaint_screen ();
