@@ -1,13 +1,13 @@
 /*
    Some misc dialog boxes for the program.
 
-   Copyright (C) 1994-2019
+   Copyright (C) 1994-2024
    Free Software Foundation, Inc.
 
    Written by:
    Miguel de Icaza, 1994, 1995
    Jakub Jelinek, 1995
-   Andrew Borodin <aborodin@vmail.ru>, 2009-2015
+   Andrew Borodin <aborodin@vmail.ru>, 2009-2022
 
    This file is part of the Midnight Commander.
 
@@ -42,6 +42,7 @@
 #include "lib/global.h"
 
 #include "lib/tty/tty.h"
+#include "lib/tty/color.h"      /* tty_use_colors() */
 #include "lib/tty/key.h"        /* XCTRL and ALT macros  */
 #include "lib/skin.h"           /* INPUT_COLOR */
 #include "lib/mcconfig.h"       /* Load/save user formats */
@@ -51,9 +52,6 @@
 #ifdef ENABLE_VFS_FTP
 #include "src/vfs/ftpfs/ftpfs.h"
 #endif /* ENABLE_VFS_FTP */
-#ifdef ENABLE_VFS_SMB
-#include "src/vfs/smbfs/smbfs.h"
-#endif /* ENABLE_VFS_SMB */
 
 #include "lib/util.h"           /* Q_() */
 #include "lib/widget.h"
@@ -72,10 +70,9 @@
 
 #include "command.h"            /* For cmdline */
 #include "dir.h"
-#include "panel.h"              /* LIST_FORMATS */
 #include "tree.h"
 #include "layout.h"             /* for get_nth_panel_name proto */
-#include "midnight.h"           /* current_panel */
+#include "filemanager.h"
 
 #include "boxes.h"
 
@@ -91,6 +88,8 @@
 
 /*** file scope type declarations ****************************************************************/
 
+/*** forward declarations (file scope functions) *************************************************/
+
 /*** file scope variables ************************************************************************/
 
 static unsigned long configure_old_esc_mode_id, configure_time_out_id;
@@ -102,7 +101,7 @@ static const int panel_list_user_idx = 3;
 
 static char **status_format;
 static unsigned long panel_list_formats_id, panel_user_format_id, panel_brief_cols_id;
-static unsigned long mini_user_status_id, mini_user_format_id;
+static unsigned long user_mini_status_id, mini_user_format_id;
 
 #ifdef HAVE_CHARSET
 static int new_display_codepage;
@@ -119,6 +118,8 @@ static gchar *current_skin_name;
 static WListbox *bg_list = NULL;
 #endif /* ENABLE_BACKGROUND */
 
+static unsigned long shadows_id;
+
 /* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
@@ -136,7 +137,7 @@ configure_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, voi
             Widget *ww;
 
             /* input line */
-            ww = dlg_find_by_id (DIALOG (w), configure_time_out_id);
+            ww = widget_find_by_id (w, configure_time_out_id);
             widget_disable (ww, not_single);
 
             return MSG_HANDLED;
@@ -190,14 +191,13 @@ skin_dlg_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
     case MSG_RESIZE:
         {
             WDialog *d = DIALOG (w);
-            Widget *wd = WIDGET (d->data);
-            int y, x;
+            const WRect *wd = &WIDGET (d->data.p)->rect;
+            WRect r = w->rect;
 
-            y = wd->y + (wd->lines - w->lines) / 2;
-            x = wd->x + wd->cols / 2;
-            dlg_set_position (d, y, x, w->lines, w->cols);
+            r.y = wd->y + (wd->lines - r.lines) / 2;
+            r.x = wd->x + wd->cols / 2;
 
-            return MSG_HANDLED;
+            return dlg_default_callback (w, NULL, MSG_RESIZE, 0, &r);
         }
 
     default:
@@ -223,7 +223,7 @@ sel_skin_button (WButton * button, int action)
         dlg_create (TRUE, 0, 0, 13, 24, WPOS_KEEP_DEFAULT, TRUE, dialog_colors, skin_dlg_callback,
                     NULL, "[Appearance]", _("Skins"));
     /* use Appearance dialog for positioning */
-    skin_dlg->data = WIDGET (button)->owner;
+    skin_dlg->data.p = WIDGET (button)->owner;
 
     /* set dialog location before all */
     send_message (skin_dlg, NULL, MSG_RESIZE, 0, NULL);
@@ -234,7 +234,7 @@ sel_skin_button (WButton * button, int action)
                       (void *) skin_name, FALSE);
 
     if (strcmp (skin_name, current_skin_name) == 0)
-        listbox_select_entry (skin_list, 0);
+        listbox_set_current (skin_list, 0);
 
     for (i = 0; i < skin_names->len; i++)
     {
@@ -244,13 +244,13 @@ sel_skin_button (WButton * button, int action)
             listbox_add_item (skin_list, LISTBOX_APPEND_AT_END, 0, skin_name_to_label (skin_name),
                               (void *) skin_name, FALSE);
             if (strcmp (skin_name, current_skin_name) == 0)
-                listbox_select_entry (skin_list, pos);
+                listbox_set_current (skin_list, pos);
             pos++;
         }
     }
 
     /* make list stick to all sides of dialog, effectively make it be resized with dialog */
-    add_widget_autopos (skin_dlg, skin_list, WPOS_KEEP_ALL, NULL);
+    group_add_widget_autopos (GROUP (skin_dlg), skin_list, WPOS_KEEP_ALL, NULL);
 
     result = dlg_run (skin_dlg);
     if (result == B_ENTER)
@@ -264,18 +264,50 @@ sel_skin_button (WButton * button, int action)
 
         button_set_text (button, str_fit_to_term (skin_label, 20, J_LEFT_FIT));
     }
-    dlg_destroy (skin_dlg);
+    widget_destroy (WIDGET (skin_dlg));
 
     return 0;
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
+static cb_ret_t
+appearance_box_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
+{
+    switch (msg)
+    {
+    case MSG_INIT:
+#ifdef ENABLE_SHADOWS
+        if (!tty_use_colors ())
+#endif
+        {
+            Widget *shadow;
+
+            shadow = widget_find_by_id (w, shadows_id);
+            CHECK (shadow)->state = FALSE;
+            widget_disable (shadow, TRUE);
+        }
+        return MSG_HANDLED;
+
+    case MSG_NOTIFY:
+        if (sender != NULL && sender->id == shadows_id)
+        {
+            mc_global.tty.shadows = CHECK (sender)->state;
+            repaint_screen ();
+            return MSG_HANDLED;
+        }
+        return MSG_NOT_HANDLED;
+
+    default:
+        return dlg_default_callback (w, sender, msg, parm, data);
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 static cb_ret_t
 panel_listing_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
-    WDialog *h = DIALOG (w);
-
     switch (msg)
     {
     case MSG_NOTIFY:
@@ -284,10 +316,10 @@ panel_listing_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm,
             WCheck *ch;
             WInput *in1, *in2, *in3;
 
-            in1 = INPUT (dlg_find_by_id (h, panel_user_format_id));
-            in2 = INPUT (dlg_find_by_id (h, panel_brief_cols_id));
-            ch = CHECK (dlg_find_by_id (h, mini_user_status_id));
-            in3 = INPUT (dlg_find_by_id (h, mini_user_format_id));
+            in1 = INPUT (widget_find_by_id (w, panel_user_format_id));
+            in2 = INPUT (widget_find_by_id (w, panel_brief_cols_id));
+            ch = CHECK (widget_find_by_id (w, user_mini_status_id));
+            in3 = INPUT (widget_find_by_id (w, mini_user_format_id));
 
             if (!ch->state)
                 input_assign_text (in3, status_format[RADIO (sender)->sel]);
@@ -299,11 +331,11 @@ panel_listing_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm,
             return MSG_HANDLED;
         }
 
-        if (sender != NULL && sender->id == mini_user_status_id)
+        if (sender != NULL && sender->id == user_mini_status_id)
         {
             WInput *in;
 
-            in = INPUT (dlg_find_by_id (h, mini_user_format_id));
+            in = INPUT (widget_find_by_id (w, mini_user_format_id));
 
             if (CHECK (sender)->state)
             {
@@ -314,7 +346,7 @@ panel_listing_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm,
             {
                 WRadio *r;
 
-                r = RADIO (dlg_find_by_id (h, panel_list_formats_id));
+                r = RADIO (widget_find_by_id (w, panel_list_formats_id));
                 widget_disable (WIDGET (in), TRUE);
                 input_assign_text (in, status_format[r->sel]);
             }
@@ -355,7 +387,7 @@ sel_charset_button (WButton * button, int action)
             cpname = _("7-bit ASCII");  /* FIXME */
 
         button_set_text (button, cpname);
-        dlg_redraw (WIDGET (button)->owner);
+        widget_draw (WIDGET (WIDGET (button)->owner));
     }
 
     return 0;
@@ -373,13 +405,16 @@ tree_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *da
     {
     case MSG_RESIZE:
         {
+            WRect r = w->rect;
             Widget *bar;
 
-            /* simply call dlg_set_size() with new size */
-            dlg_set_size (h, LINES - 9, COLS - 20);
-            bar = WIDGET (find_buttonbar (h));
-            bar->x = 0;
-            bar->y = LINES - 1;
+            r.lines = LINES - 9;
+            r.cols = COLS - 20;
+            dlg_default_callback (w, NULL, MSG_RESIZE, 0, &r);
+
+            bar = WIDGET (buttonbar_find (h));
+            bar->rect.x = 0;
+            bar->rect.y = LINES - 1;
             return MSG_HANDLED;
         }
 
@@ -407,7 +442,7 @@ confvfs_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
             Widget *wi;
 
             /* input */
-            wi = dlg_find_by_id (DIALOG (w), ftpfs_proxy_host_id);
+            wi = widget_find_by_id (w, ftpfs_proxy_host_id);
             widget_disable (wi, not_use);
             return MSG_HANDLED;
         }
@@ -484,7 +519,7 @@ task_cb (WButton * button, int action)
     jobs_fill_listbox (bg_list);
 
     /* This can be optimized to just redraw this widget :-) */
-    dlg_redraw (WIDGET (button)->owner);
+    widget_draw (WIDGET (WIDGET (button)->owner));
 
     return 0;
 }
@@ -557,9 +592,10 @@ configure_box (void)
             /* *INDENT-ON* */
         };
 
+        WRect r = { -1, -1, 0, 60 };
+
         quick_dialog_t qdlg = {
-            -1, -1, 60,
-            N_("Configure options"), "[Configuration]",
+            r, N_("Configure options"), "[Configuration]",
             quick_widgets, configure_callback, NULL
         };
 
@@ -578,7 +614,12 @@ configure_box (void)
 #endif
 
         if (quick_dialog (&qdlg) == B_ENTER)
-            old_esc_mode_timeout = atoi (time_out_new);
+        {
+            if (time_out_new[0] == '\0')
+                old_esc_mode_timeout = 0;
+            else
+                old_esc_mode_timeout = atoi (time_out_new);
+        }
 
         g_free (time_out_new);
     }
@@ -589,6 +630,8 @@ configure_box (void)
 void
 appearance_box (void)
 {
+    gboolean shadows = mc_global.tty.shadows;
+
     current_skin_name = g_strdup (mc_skin__default.name);
     skin_names = mc_skin_list ();
 
@@ -601,26 +644,31 @@ appearance_box (void)
                 QUICK_BUTTON (str_fit_to_term (skin_name_to_label (current_skin_name), 20, J_LEFT_FIT),
                               B_USER, sel_skin_button, NULL),
             QUICK_STOP_COLUMNS,
+            QUICK_SEPARATOR (TRUE),
+            QUICK_CHECKBOX (N_("&Shadows"), &mc_global.tty.shadows, &shadows_id),
             QUICK_BUTTONS_OK_CANCEL,
             QUICK_END
             /* *INDENT-ON* */
         };
 
+        WRect r = { -1, -1, 0, 54 };
+
         quick_dialog_t qdlg = {
-            -1, -1, 54,
-            N_("Appearance"), "[Appearance]",
-            quick_widgets, dlg_default_callback, NULL
+            r, N_("Appearance"), "[Appearance]",
+            quick_widgets, appearance_box_callback, NULL
         };
 
         if (quick_dialog (&qdlg) == B_ENTER)
             mc_config_set_string (mc_global.main_config, CONFIG_APP_SECTION, "skin",
                                   current_skin_name);
         else
+        {
             skin_apply (NULL);
+            mc_global.tty.shadows = shadows;
+        }
     }
 
     g_free (current_skin_name);
-    g_ptr_array_foreach (skin_names, (GFunc) g_free, NULL);
     g_ptr_array_free (skin_names, TRUE);
 }
 
@@ -632,7 +680,7 @@ panel_options_box (void)
     gboolean simple_swap;
 
     simple_swap = mc_config_get_bool (mc_global.main_config, CONFIG_PANELS_SECTION,
-                                      "simple_swap", FALSE) ? 1 : 0;
+                                      "simple_swap", FALSE);
     {
         const char *qsearch_options[] = {
             N_("Case &insensitive"),
@@ -683,9 +731,10 @@ panel_options_box (void)
             /* *INDENT-ON* */
         };
 
+        WRect r = { -1, -1, 0, 60 };
+
         quick_dialog_t qdlg = {
-            -1, -1, 60,
-            N_("Panel options"), "[Panel options]",
+            r, N_("Panel options"), "[Panel options]",
             quick_widgets, NULL, NULL
         };
 
@@ -724,7 +773,7 @@ panel_listing_box (WPanel * panel, int num, char **userp, char **minip, gboolean
     }
 
     {
-        gboolean mini_user_status;
+        gboolean user_mini_status;
         char panel_brief_cols_in[BUF_TINY];
         char *panel_brief_cols_out = NULL;
         char *panel_user_format = NULL;
@@ -751,7 +800,7 @@ panel_listing_box (WPanel * panel, int num, char **userp, char **minip, gboolean
             QUICK_INPUT (panel->user_format, "user-fmt-input", &panel_user_format,
                          &panel_user_format_id, FALSE, FALSE, INPUT_COMPLETE_NONE),
             QUICK_SEPARATOR (TRUE),
-            QUICK_CHECKBOX (N_("User &mini status"), &mini_user_status, &mini_user_status_id),
+            QUICK_CHECKBOX (N_("User &mini status"), &user_mini_status, &user_mini_status_id),
             QUICK_INPUT (panel->user_status_format[panel->list_format], "mini_input",
                          &mini_user_format, &mini_user_format_id, FALSE, FALSE, INPUT_COMPLETE_NONE),
             QUICK_BUTTONS_OK_CANCEL,
@@ -759,13 +808,14 @@ panel_listing_box (WPanel * panel, int num, char **userp, char **minip, gboolean
             /* *INDENT-ON* */
         };
 
+        WRect r = { -1, -1, 0, 48 };
+
         quick_dialog_t qdlg = {
-            -1, -1, 48,
-            N_("Listing format"), "[Listing Format...]",
+            r, N_("Listing format"), "[Listing Format...]",
             quick_widgets, panel_listing_callback, NULL
         };
 
-        mini_user_status = panel->user_mini_status;
+        user_mini_status = panel->user_mini_status;
         result = panel->list_format;
         status_format = panel->user_status_format;
 
@@ -777,7 +827,7 @@ panel_listing_box (WPanel * panel, int num, char **userp, char **minip, gboolean
         if ((int) panel->list_format != panel_list_user_idx)
             quick_widgets[6].state = WST_DISABLED;
 
-        if (!mini_user_status)
+        if (!user_mini_status)
             quick_widgets[9].state = WST_DISABLED;
 
         if (quick_dialog (&qdlg) == B_CANCEL)
@@ -789,7 +839,7 @@ panel_listing_box (WPanel * panel, int num, char **userp, char **minip, gboolean
 
             *userp = panel_user_format;
             *minip = mini_user_format;
-            *use_msformat = mini_user_status;
+            *use_msformat = user_mini_status;
 
             cols = strtol (panel_brief_cols_out, &error, 10);
             if (*error == '\0')
@@ -849,9 +899,10 @@ sort_box (dir_sort_options_t * op, const panel_field_t * sort_field)
             /* *INDENT-ON* */
         };
 
+        WRect r = { -1, -1, 0, 40 };
+
         quick_dialog_t qdlg = {
-            -1, -1, 40,
-            N_("Sort order"), "[Sort Order...]",
+            r, N_("Sort order"), "[Sort Order...]",
             quick_widgets, NULL, NULL
         };
 
@@ -888,9 +939,10 @@ confirm_box (void)
         /* *INDENT-ON* */
     };
 
+    WRect r = { -1, -1, 0, 46 };
+
     quick_dialog_t qdlg = {
-        -1, -1, 46,
-        N_("Confirmation"), "[Confirmation]",
+        r, N_("Confirmation"), "[Confirmation]",
         quick_widgets, NULL, NULL
     };
 
@@ -923,9 +975,10 @@ display_bits_box (void)
         /* *INDENT-ON* */
     };
 
+    WRect r = { -1, -1, 0, 46 };
+
     quick_dialog_t qdlg = {
-        -1, -1, 46,
-        _("Display bits"), "[Display bits]",
+        r, _("Display bits"), "[Display bits]",
         quick_widgets, NULL, NULL
     };
 
@@ -943,9 +996,9 @@ display_bits_box (void)
         mc_global.eight_bit_clean = current_mode < 3;
         mc_global.full_eight_bits = current_mode < 2;
 #ifndef HAVE_SLANG
-        meta (stdscr, mc_global.eight_bit_clean);
+        tty_display_8bit (mc_global.eight_bit_clean);
 #else
-        SLsmg_Display_Eight_Bit = mc_global.full_eight_bits ? 128 : 160;
+        tty_display_8bit (mc_global.full_eight_bits);
 #endif
         use_8th_bit_as_meta = !new_meta;
     }
@@ -981,9 +1034,10 @@ display_bits_box (void)
             /* *INDENT-ON* */
         };
 
+        WRect r = { -1, -1, 0, 46 };
+
         quick_dialog_t qdlg = {
-            -1, -1, 46,
-            N_("Display bits"), "[Display bits]",
+            r, N_("Display bits"), "[Display bits]",
             quick_widgets, NULL, NULL
         };
 
@@ -1024,6 +1078,7 @@ tree_box (const char *current_dir)
 {
     WTree *mytree;
     WDialog *dlg;
+    WGroup *g;
     Widget *wd;
     char *val = NULL;
     WButtonBar *bar;
@@ -1033,16 +1088,17 @@ tree_box (const char *current_dir)
     /* Create the components */
     dlg = dlg_create (TRUE, 0, 0, LINES - 9, COLS - 20, WPOS_CENTER, FALSE, dialog_colors,
                       tree_callback, NULL, "[Directory Tree]", _("Directory tree"));
+    g = GROUP (dlg);
     wd = WIDGET (dlg);
 
-    mytree = tree_new (2, 2, wd->lines - 6, wd->cols - 5, FALSE);
-    add_widget_autopos (dlg, mytree, WPOS_KEEP_ALL, NULL);
-    add_widget_autopos (dlg, hline_new (wd->lines - 4, 1, -1), WPOS_KEEP_BOTTOM, NULL);
-    bar = buttonbar_new (TRUE);
-    add_widget (dlg, bar);
+    mytree = tree_new (2, 2, wd->rect.lines - 6, wd->rect.cols - 5, FALSE);
+    group_add_widget_autopos (g, mytree, WPOS_KEEP_ALL, NULL);
+    group_add_widget_autopos (g, hline_new (wd->rect.lines - 4, 1, -1), WPOS_KEEP_BOTTOM, NULL);
+    bar = buttonbar_new ();
+    group_add_widget (g, bar);
     /* restore ButtonBar coordinates after add_widget() */
-    WIDGET (bar)->x = 0;
-    WIDGET (bar)->y = LINES - 1;
+    WIDGET (bar)->rect.x = 0;
+    WIDGET (bar)->rect.y = LINES - 1;
 
     if (dlg_run (dlg) == B_ENTER)
     {
@@ -1052,7 +1108,7 @@ tree_box (const char *current_dir)
         val = g_strdup (vfs_path_as_str (selected_name));
     }
 
-    dlg_destroy (dlg);
+    widget_destroy (wd);
     return val;
 }
 
@@ -1060,7 +1116,7 @@ tree_box (const char *current_dir)
 
 #ifdef ENABLE_VFS
 void
-configure_vfs (void)
+configure_vfs_box (void)
 {
     char buffer2[BUF_TINY];
 #ifdef ENABLE_VFS_FTP
@@ -1106,9 +1162,10 @@ configure_vfs (void)
             /* *INDENT-ON* */
         };
 
+        WRect r = { -1, -1, 0, 56 };
+
         quick_dialog_t qdlg = {
-            -1, -1, 56,
-            N_("Virtual File System Setting"), "[Virtual FS]",
+            r, N_("Virtual File System Setting"), "[Virtual FS]",
             quick_widgets,
 #ifdef ENABLE_VFS_FTP
             confvfs_callback,
@@ -1126,7 +1183,10 @@ configure_vfs (void)
         if (quick_dialog (&qdlg) != B_CANCEL)
         {
             /* cppcheck-suppress uninitvar */
-            vfs_timeout = atoi (ret_timeout);
+            if (ret_timeout[0] == '\0')
+                vfs_timeout = 0;
+            else
+                vfs_timeout = atoi (ret_timeout);
             g_free (ret_timeout);
 
             if (vfs_timeout < 0 || vfs_timeout > 10000)
@@ -1139,7 +1199,10 @@ configure_vfs (void)
             /* cppcheck-suppress uninitvar */
             ftpfs_proxy_host = ret_ftp_proxy;
             /* cppcheck-suppress uninitvar */
-            ftpfs_directory_timeout = atoi (ret_directory_timeout);
+            if (ret_directory_timeout[0] == '\0')
+                ftpfs_directory_timeout = 0;
+            else
+                ftpfs_directory_timeout = atoi (ret_directory_timeout);
             g_free (ret_directory_timeout);
 #endif
         }
@@ -1151,9 +1214,9 @@ configure_vfs (void)
 /* --------------------------------------------------------------------------------------------- */
 
 char *
-cd_dialog (void)
+cd_box (const WPanel * panel)
 {
-    const Widget *w = CONST_WIDGET (current_panel);
+    const Widget *w = CONST_WIDGET (panel);
     char *my_str;
 
     quick_widget_t quick_widgets[] = {
@@ -1162,9 +1225,10 @@ cd_dialog (void)
         QUICK_END
     };
 
+    WRect r = { w->rect.y + w->rect.lines - 6, w->rect.x, 0, w->rect.cols };
+
     quick_dialog_t qdlg = {
-        w->y + w->lines - 6, w->x, w->cols,
-        N_("Quick cd"), "[Quick cd]",
+        r, N_("Quick cd"), "[Quick cd]",
         quick_widgets, NULL, NULL
     };
 
@@ -1174,8 +1238,8 @@ cd_dialog (void)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-symlink_dialog (const vfs_path_t * existing_vpath, const vfs_path_t * new_vpath,
-                char **ret_existing, char **ret_new)
+symlink_box (const vfs_path_t * existing_vpath, const vfs_path_t * new_vpath,
+             char **ret_existing, char **ret_new)
 {
     quick_widget_t quick_widgets[] = {
         /* *INDENT-OFF* */
@@ -1191,9 +1255,10 @@ symlink_dialog (const vfs_path_t * existing_vpath, const vfs_path_t * new_vpath,
         /* *INDENT-ON* */
     };
 
+    WRect r = { -1, -1, 0, 64 };
+
     quick_dialog_t qdlg = {
-        -1, -1, 64,
-        N_("Symbolic link"), "[File Menu]",
+        r, N_("Symbolic link"), "[File Menu]",
         quick_widgets, NULL, NULL
     };
 
@@ -1208,7 +1273,7 @@ symlink_dialog (const vfs_path_t * existing_vpath, const vfs_path_t * new_vpath,
 
 #ifdef ENABLE_BACKGROUND
 void
-jobs_cmd (void)
+jobs_box (void)
 {
     struct
     {
@@ -1232,6 +1297,7 @@ jobs_cmd (void)
     const size_t n_but = G_N_ELEMENTS (job_but);
 
     WDialog *jobs_dlg;
+    WGroup *g;
     int cols = 60;
     int lines = 15;
     int x = 0;
@@ -1253,88 +1319,25 @@ jobs_cmd (void)
 
     jobs_dlg = dlg_create (TRUE, 0, 0, lines, cols, WPOS_CENTER, FALSE, dialog_colors, NULL, NULL,
                            "[Background jobs]", _("Background jobs"));
+    g = GROUP (jobs_dlg);
 
     bg_list = listbox_new (2, 2, lines - 6, cols - 6, FALSE, NULL);
     jobs_fill_listbox (bg_list);
-    add_widget (jobs_dlg, bg_list);
+    group_add_widget (g, bg_list);
 
-    add_widget (jobs_dlg, hline_new (lines - 4, -1, -1));
+    group_add_widget (g, hline_new (lines - 4, -1, -1));
 
     x = (cols - x) / 2;
     for (i = 0; i < n_but; i++)
     {
-        add_widget (jobs_dlg,
-                    button_new (lines - 3, x, job_but[i].value, job_but[i].flags, job_but[i].name,
-                                job_but[i].callback));
+        group_add_widget (g, button_new (lines - 3, x, job_but[i].value, job_but[i].flags,
+                                         job_but[i].name, job_but[i].callback));
         x += job_but[i].len + 1;
     }
 
     (void) dlg_run (jobs_dlg);
-    dlg_destroy (jobs_dlg);
+    widget_destroy (WIDGET (jobs_dlg));
 }
 #endif /* ENABLE_BACKGROUND */
 
 /* --------------------------------------------------------------------------------------------- */
-
-#ifdef ENABLE_VFS_SMB
-struct smb_authinfo *
-vfs_smb_get_authinfo (const char *host, const char *share, const char *domain, const char *user)
-{
-    char *label;
-    struct smb_authinfo *return_value = NULL;
-
-    if (domain == NULL)
-        domain = "";
-    if (user == NULL)
-        user = "";
-
-    label = g_strdup_printf (_("Password for \\\\%s\\%s"), host, share);
-
-    {
-        char *ret_domain, *ret_user, *ret_password;
-
-        quick_widget_t quick_widgets[] = {
-            /* *INDENT-OFF* */
-            QUICK_LABEL (label, NULL),
-            QUICK_SEPARATOR (TRUE),
-            QUICK_START_COLUMNS,
-                QUICK_LABEL (N_("Domain:"), NULL),
-                QUICK_SEPARATOR (FALSE),
-                QUICK_LABEL (N_("Username:"), NULL),
-                QUICK_SEPARATOR (FALSE),
-                QUICK_LABEL (N_("Password:"), NULL),
-            QUICK_NEXT_COLUMN,
-                QUICK_INPUT (domain, "auth_domain", &ret_domain, NULL, FALSE, FALSE, INPUT_COMPLETE_HOSTNAMES),
-                QUICK_SEPARATOR (FALSE),
-                QUICK_INPUT (user, "auth_name", &ret_user, NULL, FALSE, FALSE, INPUT_COMPLETE_USERNAMES),
-                QUICK_SEPARATOR (FALSE),
-                QUICK_INPUT ("", "auth_password", &ret_password, NULL, TRUE, FALSE, INPUT_COMPLETE_NONE),
-            QUICK_STOP_COLUMNS,
-            QUICK_BUTTONS_OK_CANCEL,
-            QUICK_END
-            /* *INDENT-ON* */
-        };
-
-        quick_dialog_t qdlg = {
-            -1, -1, 40,
-            N_("SMB authentication"), "[Smb Authinfo]",
-            quick_widgets, NULL, NULL
-        };
-
-        if (quick_dialog (&qdlg) != B_CANCEL)
-        {
-            return_value = vfs_smb_authinfo_new (host, share, ret_domain, ret_user, ret_password);
-
-            g_free (ret_domain);
-            g_free (ret_user);
-            g_free (ret_password);
-        }
-    }
-
-    g_free (label);
-
-    return return_value;
-}
-#endif /* ENABLE_VFS_SMB */
-
-/* --------------------------------------------------------------------------------------------- */
