@@ -1,7 +1,7 @@
 /*
    Chmod command -- for the Midnight Commander
 
-   Copyright (C) 1994-2019
+   Copyright (C) 1994-2024
    Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
@@ -40,9 +40,7 @@
 #include "lib/util.h"
 #include "lib/widget.h"
 
-#include "midnight.h"           /* current_panel */
-
-#include "chmod.h"
+#include "cmd.h"                /* chmod_cmd() */
 
 /*** global variables ****************************************************************************/
 
@@ -62,6 +60,8 @@
 
 /*** file scope type declarations ****************************************************************/
 
+/*** forward declarations (file scope functions) *************************************************/
+
 /*** file scope variables ************************************************************************/
 
 static struct
@@ -132,11 +132,14 @@ static WGroupbox *file_gb;
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-chmod_i18n (void)
+chmod_init (void)
 {
     static gboolean i18n = FALSE;
     int i, len;
 
+    for (i = 0; i < BUTTONS_PERM; i++)
+        check_perm[i].selected = FALSE;
+
     if (i18n)
         return;
 
@@ -178,28 +181,39 @@ chmod_i18n (void)
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-chmod_toggle_select (WDialog * h, int Id)
+chmod_draw_select (const WDialog * h, int Id)
 {
-    tty_setcolor (COLOR_NORMAL);
-    check_perm[Id].selected = !check_perm[Id].selected;
-
-    widget_move (h, PY + Id + 1, PX + 1);
+    widget_gotoyx (h, PY + Id + 1, PX + 1);
     tty_print_char (check_perm[Id].selected ? '*' : ' ');
-    widget_move (h, PY + Id + 1, PX + 3);
+    widget_gotoyx (h, PY + Id + 1, PX + 3);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-chmod_refresh (WDialog * h)
+chmod_toggle_select (const WDialog * h, int Id)
 {
-    int y = WIDGET (file_gb)->y + 1;
-    int x = WIDGET (file_gb)->x + 2;
+    check_perm[Id].selected = !check_perm[Id].selected;
+    tty_setcolor (COLOR_NORMAL);
+    chmod_draw_select (h, Id);
+}
 
-    dlg_default_repaint (h);
+/* --------------------------------------------------------------------------------------------- */
+
+static void
+chmod_refresh (const WDialog * h)
+{
+    int i;
+    int y, x;
 
     tty_setcolor (COLOR_NORMAL);
 
+    for (i = 0; i < BUTTONS_PERM; i++)
+        chmod_draw_select (h, i);
+
+    y = WIDGET (file_gb)->rect.y + 1;
+    x = WIDGET (file_gb)->rect.x + 2;
+
     tty_gotoyx (y, x);
     tty_print_string (file_info_labels[0]);
     tty_gotoyx (y + 2, x);
@@ -212,17 +226,31 @@ chmod_refresh (WDialog * h)
 
 /* --------------------------------------------------------------------------------------------- */
 
+static cb_ret_t
+chmod_bg_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
+{
+    switch (msg)
+    {
+    case MSG_DRAW:
+        frame_callback (w, NULL, MSG_DRAW, 0, NULL);
+        chmod_refresh (CONST_DIALOG (w->owner));
+        return MSG_HANDLED;
+
+    default:
+        return frame_callback (w, sender, msg, parm, data);
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 static cb_ret_t
 chmod_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
+    WGroup *g = GROUP (w);
     WDialog *h = DIALOG (w);
 
     switch (msg)
     {
-    case MSG_DRAW:
-        chmod_refresh (h);
-        return MSG_HANDLED;
-
     case MSG_NOTIFY:
         {
             /* handle checkboxes */
@@ -235,11 +263,8 @@ chmod_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
 
             if (i < BUTTONS_PERM)
             {
-                char buffer[BUF_TINY];
-
                 ch_mode ^= check_perm[i].mode;
-                g_snprintf (buffer, sizeof (buffer), "%o", (unsigned int) ch_mode);
-                label_set_text (statl, buffer);
+                label_set_textv (statl, "%o", (unsigned int) ch_mode);
                 chmod_toggle_select (h, i);
                 mode_change = TRUE;
                 return MSG_HANDLED;
@@ -254,7 +279,7 @@ chmod_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
             int i;
             unsigned long id;
 
-            id = dlg_get_current_widget_id (h);
+            id = group_get_current_widget_id (g);
             for (i = 0; i < BUTTONS_PERM; i++)
                 if (id == WIDGET (check_perm[i].check)->id)
                     break;
@@ -263,7 +288,7 @@ chmod_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
             {
                 chmod_toggle_select (h, i);
                 if (parm == KEY_IC)
-                    dlg_select_next_widget (h);
+                    group_select_next_widget (g);
                 return MSG_HANDLED;
             }
         }
@@ -277,10 +302,11 @@ chmod_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
 /* --------------------------------------------------------------------------------------------- */
 
 static WDialog *
-chmod_init (const char *fname, const struct stat *sf_stat)
+chmod_dlg_create (WPanel * panel, const char *fname, const struct stat *sf_stat)
 {
     gboolean single_set;
     WDialog *ch_dlg;
+    WGroup *g;
     int lines, cols;
     int i, y;
     int perm_gb_len;
@@ -290,7 +316,7 @@ chmod_init (const char *fname, const struct stat *sf_stat)
 
     mode_change = FALSE;
 
-    single_set = (current_panel->marked < 2);
+    single_set = (panel->marked < 2);
     perm_gb_len = check_perm_len + 2;
     file_gb_len = file_info_labels_len + 2;
     cols = str_term_width1 (fname) + 2 + 1;
@@ -309,61 +335,64 @@ chmod_init (const char *fname, const struct stat *sf_stat)
     ch_dlg =
         dlg_create (TRUE, 0, 0, lines, cols, WPOS_CENTER, FALSE, dialog_colors,
                     chmod_callback, NULL, "[Chmod]", _("Chmod command"));
+    g = GROUP (ch_dlg);
 
-    add_widget (ch_dlg, groupbox_new (PY, PX, BUTTONS_PERM + 2, perm_gb_len, _("Permission")));
+    /* draw background */
+    ch_dlg->bg->callback = chmod_bg_callback;
+
+    group_add_widget (g, groupbox_new (PY, PX, BUTTONS_PERM + 2, perm_gb_len, _("Permission")));
 
     for (i = 0; i < BUTTONS_PERM; i++)
     {
         check_perm[i].check = check_new (PY + i + 1, PX + 2, (ch_mode & check_perm[i].mode) != 0,
                                          check_perm[i].text);
-        add_widget (ch_dlg, check_perm[i].check);
+        group_add_widget (g, check_perm[i].check);
     }
 
     file_gb = groupbox_new (PY, PX + perm_gb_len + 1, BUTTONS_PERM + 2, file_gb_len, _("File"));
-    add_widget (ch_dlg, file_gb);
+    group_add_widget (g, file_gb);
 
     /* Set the labels */
     y = PY + 2;
     cols = PX + perm_gb_len + 3;
     c_fname = str_trunc (fname, file_gb_len - 3);
-    add_widget (ch_dlg, label_new (y, cols, c_fname));
+    group_add_widget (g, label_new (y, cols, c_fname));
     g_snprintf (buffer, sizeof (buffer), "%o", (unsigned int) ch_mode);
     statl = label_new (y + 2, cols, buffer);
-    add_widget (ch_dlg, statl);
+    group_add_widget (g, statl);
     c_fown = str_trunc (get_owner (sf_stat->st_uid), file_gb_len - 3);
-    add_widget (ch_dlg, label_new (y + 4, cols, c_fown));
+    group_add_widget (g, label_new (y + 4, cols, c_fown));
     c_fgrp = str_trunc (get_group (sf_stat->st_gid), file_gb_len - 3);
-    add_widget (ch_dlg, label_new (y + 6, cols, c_fgrp));
+    group_add_widget (g, label_new (y + 6, cols, c_fgrp));
 
     if (!single_set)
     {
         i = 0;
-        add_widget (ch_dlg, hline_new (lines - chmod_but[i].y - 1, -1, -1));
+
+        group_add_widget (g, hline_new (lines - chmod_but[i].y - 1, -1, -1));
+
         for (; i < BUTTONS - 2; i++)
         {
             y = lines - chmod_but[i].y;
-            add_widget (ch_dlg,
-                        button_new (y, WIDGET (ch_dlg)->cols / 2 - chmod_but[i].len,
-                                    chmod_but[i].ret_cmd, chmod_but[i].flags, chmod_but[i].text,
-                                    NULL));
+            group_add_widget (g, button_new (y, WIDGET (ch_dlg)->rect.cols / 2 - chmod_but[i].len,
+                                             chmod_but[i].ret_cmd, chmod_but[i].flags,
+                                             chmod_but[i].text, NULL));
             i++;
-            add_widget (ch_dlg,
-                        button_new (y, WIDGET (ch_dlg)->cols / 2 + 1,
-                                    chmod_but[i].ret_cmd, chmod_but[i].flags, chmod_but[i].text,
-                                    NULL));
+            group_add_widget (g, button_new (y, WIDGET (ch_dlg)->rect.cols / 2 + 1,
+                                             chmod_but[i].ret_cmd, chmod_but[i].flags,
+                                             chmod_but[i].text, NULL));
         }
     }
 
     i = BUTTONS - 2;
     y = lines - chmod_but[i].y;
-    add_widget (ch_dlg, hline_new (y - 1, -1, -1));
-    add_widget (ch_dlg,
-                button_new (y, WIDGET (ch_dlg)->cols / 2 - chmod_but[i].len, chmod_but[i].ret_cmd,
-                            chmod_but[i].flags, chmod_but[i].text, NULL));
+    group_add_widget (g, hline_new (y - 1, -1, -1));
+    group_add_widget (g, button_new (y, WIDGET (ch_dlg)->rect.cols / 2 - chmod_but[i].len,
+                                     chmod_but[i].ret_cmd, chmod_but[i].flags, chmod_but[i].text,
+                                     NULL));
     i++;
-    add_widget (ch_dlg,
-                button_new (y, WIDGET (ch_dlg)->cols / 2 + 1, chmod_but[i].ret_cmd,
-                            chmod_but[i].flags, chmod_but[i].text, NULL));
+    group_add_widget (g, button_new (y, WIDGET (ch_dlg)->rect.cols / 2 + 1, chmod_but[i].ret_cmd,
+                                     chmod_but[i].flags, chmod_but[i].text, NULL));
 
     /* select first checkbox */
     widget_select (WIDGET (check_perm[0].check));
@@ -383,13 +412,13 @@ chmod_done (gboolean need_update)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static const char *
-next_file (void)
+static const GString *
+next_file (const WPanel * panel)
 {
-    while (!current_panel->dir.list[current_file].f.marked)
+    while (panel->dir.list[current_file].f.marked == 0)
         current_file++;
 
-    return current_panel->dir.list[current_file].fname;
+    return panel->dir.list[current_file].fname;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -397,15 +426,17 @@ next_file (void)
 static gboolean
 try_chmod (const vfs_path_t * p, mode_t m)
 {
+    const char *fname = NULL;
+
     while (mc_chmod (p, m) == -1 && !ignore_all)
     {
         int my_errno = errno;
         int result;
         char *msg;
 
-        msg =
-            g_strdup_printf (_("Cannot chmod \"%s\"\n%s"), x_basename (vfs_path_as_str (p)),
-                             unix_error_string (my_errno));
+        if (fname == NULL)
+            fname = x_basename (vfs_path_as_str (p));
+        msg = g_strdup_printf (_("Cannot chmod \"%s\"\n%s"), fname, unix_error_string (my_errno));
         result =
             query_dialog (MSG_ERROR, msg, D_ERROR, 4, _("&Ignore"), _("Ignore &all"), _("&Retry"),
                           _("&Cancel"));
@@ -439,7 +470,7 @@ try_chmod (const vfs_path_t * p, mode_t m)
 /* --------------------------------------------------------------------------------------------- */
 
 static gboolean
-do_chmod (const vfs_path_t * p, struct stat *sf)
+do_chmod (WPanel * panel, const vfs_path_t * p, struct stat *sf)
 {
     gboolean ret;
 
@@ -448,7 +479,7 @@ do_chmod (const vfs_path_t * p, struct stat *sf)
 
     ret = try_chmod (p, sf->st_mode);
 
-    do_file_mark (current_panel, current_file, 0);
+    do_file_mark (panel, current_file, 0);
 
     return ret;
 }
@@ -456,26 +487,26 @@ do_chmod (const vfs_path_t * p, struct stat *sf)
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-apply_mask (vfs_path_t * vpath, struct stat *sf)
+apply_mask (WPanel * panel, vfs_path_t * vpath, struct stat *sf)
 {
     gboolean ok;
 
-    if (!do_chmod (vpath, sf))
+    if (!do_chmod (panel, vpath, sf))
         return;
 
     do
     {
-        const char *fname;
+        const GString *fname;
 
-        fname = next_file ();
-        vpath = vfs_path_from_str (fname);
+        fname = next_file (panel);
+        vpath = vfs_path_from_str (fname->str);
         ok = (mc_stat (vpath, sf) == 0);
 
         if (!ok)
         {
             /* if current file was deleted outside mc -- try next file */
-            /* decrease current_panel->marked */
-            do_file_mark (current_panel, current_file, 0);
+            /* decrease panel->marked */
+            do_file_mark (panel, current_file, 0);
 
             /* try next file */
             ok = TRUE;
@@ -484,12 +515,12 @@ apply_mask (vfs_path_t * vpath, struct stat *sf)
         {
             ch_mode = sf->st_mode;
 
-            ok = do_chmod (vpath, sf);
+            ok = do_chmod (panel, vpath, sf);
         }
 
-        vfs_path_free (vpath);
+        vfs_path_free (vpath, TRUE);
     }
-    while (ok && current_panel->marked != 0);
+    while (ok && panel->marked != 0);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -497,12 +528,12 @@ apply_mask (vfs_path_t * vpath, struct stat *sf)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-chmod_cmd (void)
+chmod_cmd (WPanel * panel)
 {
     gboolean need_update;
     gboolean end_chmod;
 
-    chmod_i18n ();
+    chmod_init ();
 
     current_file = 0;
     ignore_all = FALSE;
@@ -512,7 +543,7 @@ chmod_cmd (void)
         vfs_path_t *vpath;
         WDialog *ch_dlg;
         struct stat sf_stat;
-        const char *fname;
+        const GString *fname;
         int i, result;
 
         do_refresh ();
@@ -520,23 +551,22 @@ chmod_cmd (void)
         need_update = FALSE;
         end_chmod = FALSE;
 
-        if (current_panel->marked != 0)
-            fname = next_file ();       /* next marked file */
+        if (panel->marked != 0)
+            fname = next_file (panel);  /* next marked file */
         else
-            fname = selection (current_panel)->fname;   /* single file */
+            fname = panel_current_entry (panel)->fname; /* single file */
 
-        vpath = vfs_path_from_str (fname);
+        vpath = vfs_path_from_str (fname->str);
 
         if (mc_stat (vpath, &sf_stat) != 0)
         {
-            vfs_path_free (vpath);
+            vfs_path_free (vpath, TRUE);
             break;
         }
 
         ch_mode = sf_stat.st_mode;
 
-        ch_dlg = chmod_init (fname, &sf_stat);
-
+        ch_dlg = chmod_dlg_create (panel, fname->str, &sf_stat);
         result = dlg_run (ch_dlg);
 
         switch (result)
@@ -548,11 +578,11 @@ chmod_cmd (void)
         case B_ENTER:
             if (mode_change)
             {
-                if (current_panel->marked <= 1)
+                if (panel->marked <= 1)
                 {
                     /* single or last file */
                     if (mc_chmod (vpath, ch_mode) == -1 && !ignore_all)
-                        message (D_ERROR, MSG_ERROR, _("Cannot chmod \"%s\"\n%s"), fname,
+                        message (D_ERROR, MSG_ERROR, _("Cannot chmod \"%s\"\n%s"), fname->str,
                                  unix_error_string (errno));
                     end_chmod = TRUE;
                 }
@@ -581,7 +611,7 @@ chmod_cmd (void)
                         and_mask &= ~check_perm[i].mode;
                 }
 
-            apply_mask (vpath, &sf_stat);
+            apply_mask (panel, vpath, &sf_stat);
             need_update = TRUE;
             end_chmod = TRUE;
             break;
@@ -594,7 +624,7 @@ chmod_cmd (void)
                 if (check_perm[i].selected)
                     or_mask |= check_perm[i].mode;
 
-            apply_mask (vpath, &sf_stat);
+            apply_mask (panel, vpath, &sf_stat);
             need_update = TRUE;
             end_chmod = TRUE;
             break;
@@ -607,7 +637,7 @@ chmod_cmd (void)
                 if (check_perm[i].selected)
                     and_mask &= ~check_perm[i].mode;
 
-            apply_mask (vpath, &sf_stat);
+            apply_mask (panel, vpath, &sf_stat);
             need_update = TRUE;
             end_chmod = TRUE;
             break;
@@ -616,17 +646,17 @@ chmod_cmd (void)
             break;
         }
 
-        if (current_panel->marked != 0 && result != B_CANCEL)
+        if (panel->marked != 0 && result != B_CANCEL)
         {
-            do_file_mark (current_panel, current_file, 0);
+            do_file_mark (panel, current_file, 0);
             need_update = TRUE;
         }
 
-        vfs_path_free (vpath);
+        vfs_path_free (vpath, TRUE);
 
-        dlg_destroy (ch_dlg);
+        widget_destroy (WIDGET (ch_dlg));
     }
-    while (current_panel->marked != 0 && !end_chmod);
+    while (panel->marked != 0 && !end_chmod);
 
     chmod_done (need_update);
 }
