@@ -10,7 +10,7 @@
    Janne Kukonlehto added much error recovery to them for being used
    in an interactive program.
 
-   Copyright (C) 1994-2019
+   Copyright (C) 1994-2024
    Free Software Foundation, Inc.
 
    Written by:
@@ -21,7 +21,7 @@
    Norbert Warmuth, 1997
    Pavel Machek, 1998
    Slava Zanko, 2009, 2010, 2011, 2012, 2013
-   Andrew Borodin <aborodin@vmail.ru>, 2009, 2010, 2011, 2012, 2013
+   Andrew Borodin <aborodin@vmail.ru>, 2009-2023
 
    This file is part of the Midnight Commander.
 
@@ -90,7 +90,7 @@
 #define STRUCT_STATVFS struct statvfs
 #define STATFS statvfs
 
-#if __linux__ && (__GLIBC__ || __UCLIBC__)
+#if defined __linux__ && (defined __GLIBC__ || defined __UCLIBC__)
 #include <sys/utsname.h>
 #include <sys/statfs.h>
 #define STAT_STATFS2_BSIZE 1
@@ -155,7 +155,7 @@ statfs (char const *filename, struct fs_info *buf)
 
 #include "src/setup.h"          /* verbose, safe_overwrite */
 
-#include "midnight.h"
+#include "filemanager.h"
 #include "fileopctx.h"          /* FILE_CONT */
 
 #include "filegui.h"
@@ -168,8 +168,8 @@ gboolean classic_progressbar = TRUE;
 
 /*** file scope macro definitions ****************************************************************/
 
-#define truncFileString(dlg, s)       str_trunc (s, WIDGET (dlg)->cols - 10)
-#define truncFileStringSecure(dlg, s) path_trunc (s, WIDGET (dlg)->cols - 10)
+#define truncFileString(dlg, s)       str_trunc (s, WIDGET (dlg)->rect.cols - 10)
+#define truncFileStringSecure(dlg, s) path_trunc (s, WIDGET (dlg)->rect.cols - 10)
 
 /*** file scope type declarations ****************************************************************/
 
@@ -237,6 +237,8 @@ typedef struct
     struct stat *src_stat, *dst_stat;
 } file_op_context_ui_t;
 
+/*** forward declarations (file scope functions) *************************************************/
+
 /*** file scope variables ************************************************************************/
 
 static struct
@@ -269,7 +271,7 @@ static struct
 static int
 statvfs_works (void)
 {
-#if ! (__linux__ && (__GLIBC__ || __UCLIBC__))
+#if ! (defined __linux__ && (defined __GLIBC__ || defined __UCLIBC__))
     return 1;
 #else
     static int statvfs_works_cache = -1;
@@ -341,7 +343,7 @@ file_frmt_time (char *buffer, double eta_secs)
     eta_hours = (int) (eta_secs / (60 * 60));
     eta_mins = (int) ((eta_secs - (eta_hours * 60 * 60)) / 60);
     eta_s = (int) (eta_secs - (eta_hours * 60 * 60 + eta_mins * 60));
-    g_snprintf (buffer, BUF_TINY, _("%d:%02d.%02d"), eta_hours, eta_mins, eta_s);
+    g_snprintf (buffer, BUF_TINY, _("%d:%02d:%02d"), eta_hours, eta_mins, eta_s);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -380,6 +382,27 @@ file_bps_prepare_for_show (char *buffer, long bps)
 
 /* --------------------------------------------------------------------------------------------- */
 
+static cb_ret_t
+file_ui_op_dlg_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
+{
+    switch (msg)
+    {
+    case MSG_ACTION:
+        /* Do not close the dialog because the query dialog will be shown */
+        if (parm == CK_Cancel)
+        {
+            DIALOG (w)->ret_value = FILE_ABORT; /* for check_progress_buttons() */
+            return MSG_HANDLED;
+        }
+        return MSG_NOT_HANDLED;
+
+    default:
+        return dlg_default_callback (w, sender, msg, parm, data);
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 /* The dialog layout:
  *
  * +---------------------- File exists -----------------------+
@@ -392,9 +415,10 @@ file_bps_prepare_for_show (char *buffer, long bps)
  * |            [ Yes ] [ No ] [ Append ] [ Reget ]           |   // 9, 10, 11, 12
  * +----------------------------------------------------------+
  * |                   Overwrite all files?                   |   // 13
- * |  [ All ] [ Older ] [None] [ Smaller ] [ Size differs ]   |   // 14, 15, 16, 17, 18
+ * |  [ ] Don't overwrite with zero length file               |   // 14
+ * |  [ All ] [ Older ] [None] [ Smaller ] [ Size differs ]   |   // 15, 16, 17, 18, 19
  * +----------------------------------------------------------|
- * |                         [ Abort ]                        |   // 19
+ * |                         [ Abort ]                        |   // 20
  * +----------------------------------------------------------+
  */
 
@@ -402,23 +426,23 @@ static replace_action_t
 overwrite_query_dialog (file_op_context_t * ctx, enum OperationMode mode)
 {
 #define W(i) dlg_widgets[i].widget
-#define WX(i) W(i)->x
-#define WCOLS(i) W(i)->cols
+#define WX(i) W(i)->rect.x
+#define WY(i) W(i)->rect.y
+#define WCOLS(i) W(i)->rect.cols
 
 #define NEW_LABEL(i, text) \
     W(i) = WIDGET (label_new (dlg_widgets[i].y, dlg_widgets[i].x, text))
 
 #define ADD_LABEL(i) \
-    add_widget_autopos (ui->replace_dlg, W(i), dlg_widgets[i].pos_flags, \
-                        ui->replace_dlg->current != NULL ? ui->replace_dlg->current->data : NULL)
+    group_add_widget_autopos (g, W(i), dlg_widgets[i].pos_flags, \
+                              g->current != NULL ? g->current->data : NULL)
 
 #define NEW_BUTTON(i) \
     W(i) = WIDGET (button_new (dlg_widgets[i].y, dlg_widgets[i].x, \
                                dlg_widgets[i].value, NORMAL_BUTTON, dlg_widgets[i].text, NULL))
 
 #define ADD_BUTTON(i) \
-    add_widget_autopos (ui->replace_dlg, W(i), dlg_widgets[i].pos_flags, \
-                        ui->replace_dlg->current->data)
+    group_add_widget_autopos (g, W(i), dlg_widgets[i].pos_flags, g->current->data)
 
     /* dialog sizes */
     const int dlg_height = 17;
@@ -487,10 +511,12 @@ overwrite_query_dialog (file_op_context_t * ctx, enum OperationMode mode)
 
     file_op_context_ui_t *ui = ctx->ui;
     Widget *wd;
+    WGroup *g;
     const char *title;
 
     vfs_path_t *p;
     char *s1;
+    const char *cs1;
     char s2[BUF_SMALL];
     int w, bw1, bw2;
     unsigned short i;
@@ -521,14 +547,14 @@ overwrite_query_dialog (file_op_context_t * ctx, enum OperationMode mode)
     p = vfs_path_from_str (ui->src_filename);
     s1 = vfs_path_to_str_flags (p, 0, VPF_STRIP_HOME | VPF_STRIP_PASSWORD);
     NEW_LABEL (1, s1);
-    vfs_path_free (p);
+    vfs_path_free (p, TRUE);
     g_free (s1);
     /* new file size */
     size_trunc_len (s2, sizeof (s2), ui->src_stat->st_size, 0, panels_options.kilobyte_si);
     NEW_LABEL (2, s2);
     /* new file modification date & time */
-    s1 = (char *) file_date (ui->src_stat->st_mtime);
-    NEW_LABEL (3, s1);
+    cs1 = file_date (ui->src_stat->st_mtime);
+    NEW_LABEL (3, cs1);
 
     /* existing file */
     NEW_LABEL (4, dlg_widgets[4].text);
@@ -536,14 +562,14 @@ overwrite_query_dialog (file_op_context_t * ctx, enum OperationMode mode)
     p = vfs_path_from_str (ui->tgt_filename);
     s1 = vfs_path_to_str_flags (p, 0, VPF_STRIP_HOME | VPF_STRIP_PASSWORD);
     NEW_LABEL (5, s1);
-    vfs_path_free (p);
+    vfs_path_free (p, TRUE);
     g_free (s1);
     /* existing file size */
     size_trunc_len (s2, sizeof (s2), ui->dst_stat->st_size, 0, panels_options.kilobyte_si);
     NEW_LABEL (6, s2);
     /* existing file modification date & time */
-    s1 = (char *) file_date (ui->dst_stat->st_mtime);
-    NEW_LABEL (7, s1);
+    cs1 = file_date (ui->dst_stat->st_mtime);
+    NEW_LABEL (7, cs1);
 
     /* will "Append" and "Reget" buttons be in the dialog? */
     do_append = !S_ISDIR (ui->dst_stat->st_mode);
@@ -636,11 +662,12 @@ overwrite_query_dialog (file_op_context_t * ctx, enum OperationMode mode)
         dlg_create (TRUE, 0, 0, dlg_height, dlg_width, WPOS_CENTER, FALSE, alarm_colors, NULL, NULL,
                     "[Replace]", title);
     wd = WIDGET (ui->replace_dlg);
+    g = GROUP (ui->replace_dlg);
 
     /* file info */
     for (i = 0; i <= 7; i++)
         ADD_LABEL (i);
-    add_widget (ui->replace_dlg, hline_new (W (7)->y - wd->y + 1, -1, -1));
+    group_add_widget (g, hline_new (WY (7) - wd->rect.y + 1, -1, -1));
 
     /* label & buttons */
     ADD_LABEL (8);              /* Overwrite this file? */
@@ -650,25 +677,25 @@ overwrite_query_dialog (file_op_context_t * ctx, enum OperationMode mode)
         ADD_BUTTON (11);        /* Append */
     if (do_reget)
         ADD_BUTTON (12);        /* Reget */
-    add_widget (ui->replace_dlg, hline_new (W (10)->y - wd->y + 1, -1, -1));
+    group_add_widget (g, hline_new (WY (10) - wd->rect.y + 1, -1, -1));
 
     /* label & buttons */
     ADD_LABEL (13);             /* Overwrite all files? */
-    add_widget (ui->replace_dlg, dlg_widgets[14].widget);
+    group_add_widget (g, dlg_widgets[14].widget);
     for (i = 15; i <= 19; i++)
         ADD_BUTTON (i);
-    add_widget (ui->replace_dlg, hline_new (W (19)->y - wd->y + 1, -1, -1));
+    group_add_widget (g, hline_new (WY (19) - wd->rect.y + 1, -1, -1));
 
     ADD_BUTTON (20);            /* Abort */
 
-    dlg_select_by_id (ui->replace_dlg, safe_overwrite ? no_id : yes_id);
+    group_select_widget_by_id (g, safe_overwrite ? no_id : yes_id);
 
     result = dlg_run (ui->replace_dlg);
 
     if (result != B_CANCEL)
         ui->dont_overwrite_with_zero = CHECK (dlg_widgets[14].widget)->state;
 
-    dlg_destroy (ui->replace_dlg);
+    widget_destroy (wd);
 
     return (result == B_CANCEL) ? REPLACE_ABORT : (replace_action_t) result;
 
@@ -719,9 +746,9 @@ place_progress_buttons (WDialog * h, gboolean suspended)
     buttons_width += progress_buttons[i].len;
     button_set_text (BUTTON (progress_buttons[i].w), progress_buttons[i].text);
 
-    progress_buttons[0].w->x = w->x + (w->cols - buttons_width) / 2;
-    progress_buttons[i].w->x = progress_buttons[0].w->x + progress_buttons[0].len + 1;
-    progress_buttons[3].w->x = progress_buttons[i].w->x + progress_buttons[i].len + 1;
+    progress_buttons[0].w->rect.x = w->rect.x + (w->rect.cols - buttons_width) / 2;
+    progress_buttons[i].w->rect.x = progress_buttons[0].w->rect.x + progress_buttons[0].len + 1;
+    progress_buttons[3].w->rect.x = progress_buttons[i].w->rect.x + progress_buttons[i].len + 1;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -769,7 +796,7 @@ check_progress_buttons (file_op_context_t * ctx)
         {
             /* redraw dialog in case of Skip after Suspend */
             place_progress_buttons (ui->op_dlg, FALSE);
-            dlg_redraw (ui->op_dlg);
+            widget_draw (WIDGET (ui->op_dlg));
         }
         ctx->suspended = FALSE;
         return FILE_SKIP;
@@ -780,7 +807,7 @@ check_progress_buttons (file_op_context_t * ctx)
     case FILE_SUSPEND:
         ctx->suspended = !ctx->suspended;
         place_progress_buttons (ui->op_dlg, ctx->suspended);
-        dlg_redraw (ui->op_dlg);
+        widget_draw (WIDGET (ui->op_dlg));
         MC_FALLTHROUGH;
     default:
         if (ctx->suspended)
@@ -797,9 +824,12 @@ file_op_context_create_ui (file_op_context_t * ctx, gboolean with_eta,
                            filegui_dialog_type_t dialog_type)
 {
     file_op_context_ui_t *ui;
+    Widget *w;
+    WGroup *g;
     int buttons_width;
     int dlg_width = 58, dlg_height = 17;
     int y = 2, x = 3;
+    WRect r;
 
     if (ctx == NULL || ctx->ui != NULL)
         return;
@@ -821,40 +851,40 @@ file_op_context_create_ui (file_op_context_t * ctx, gboolean with_eta,
     ui = ctx->ui;
     ui->replace_result = REPLACE_YES;
 
-    ui->op_dlg =
-        dlg_create (TRUE, 0, 0, dlg_height, dlg_width, WPOS_CENTER, FALSE, dialog_colors, NULL,
-                    NULL, NULL, op_names[ctx->operation]);
+    ui->op_dlg = dlg_create (TRUE, 0, 0, dlg_height, dlg_width, WPOS_CENTER, FALSE, dialog_colors,
+                             file_ui_op_dlg_callback, NULL, NULL, op_names[ctx->operation]);
+    w = WIDGET (ui->op_dlg);
+    g = GROUP (ui->op_dlg);
 
     if (dialog_type != FILEGUI_DIALOG_DELETE_ITEM)
     {
         ui->showing_eta = with_eta && ctx->progress_totals_computed;
         ui->showing_bps = with_eta;
 
-        ui->src_file_label = label_new (y++, x, "");
-        add_widget (ui->op_dlg, ui->src_file_label);
+        ui->src_file_label = label_new (y++, x, NULL);
+        group_add_widget (g, ui->src_file_label);
 
-        ui->src_file = label_new (y++, x, "");
-        add_widget (ui->op_dlg, ui->src_file);
+        ui->src_file = label_new (y++, x, NULL);
+        group_add_widget (g, ui->src_file);
 
-        ui->tgt_file_label = label_new (y++, x, "");
-        add_widget (ui->op_dlg, ui->tgt_file_label);
+        ui->tgt_file_label = label_new (y++, x, NULL);
+        group_add_widget (g, ui->tgt_file_label);
 
-        ui->tgt_file = label_new (y++, x, "");
-        add_widget (ui->op_dlg, ui->tgt_file);
+        ui->tgt_file = label_new (y++, x, NULL);
+        group_add_widget (g, ui->tgt_file);
 
         ui->progress_file_gauge = gauge_new (y++, x + 3, dlg_width - (x + 3) * 2, FALSE, 100, 0);
         if (!classic_progressbar && (current_panel == right_panel))
             ui->progress_file_gauge->from_left_to_right = FALSE;
-        add_widget_autopos (ui->op_dlg, ui->progress_file_gauge, WPOS_KEEP_TOP | WPOS_KEEP_HORZ,
-                            NULL);
+        group_add_widget_autopos (g, ui->progress_file_gauge, WPOS_KEEP_TOP | WPOS_KEEP_HORZ, NULL);
 
-        ui->progress_file_label = label_new (y++, x, "");
-        add_widget (ui->op_dlg, ui->progress_file_label);
+        ui->progress_file_label = label_new (y++, x, NULL);
+        group_add_widget (g, ui->progress_file_label);
 
         if (verbose && dialog_type == FILEGUI_DIALOG_MULTI_ITEM)
         {
             ui->total_bytes_label = hline_new (y++, -1, -1);
-            add_widget (ui->op_dlg, ui->total_bytes_label);
+            group_add_widget (g, ui->total_bytes_label);
 
             if (ctx->progress_totals_computed)
             {
@@ -862,27 +892,27 @@ file_op_context_create_ui (file_op_context_t * ctx, gboolean with_eta,
                     gauge_new (y++, x + 3, dlg_width - (x + 3) * 2, FALSE, 100, 0);
                 if (!classic_progressbar && (current_panel == right_panel))
                     ui->progress_total_gauge->from_left_to_right = FALSE;
-                add_widget_autopos (ui->op_dlg, ui->progress_total_gauge,
-                                    WPOS_KEEP_TOP | WPOS_KEEP_HORZ, NULL);
+                group_add_widget_autopos (g, ui->progress_total_gauge,
+                                          WPOS_KEEP_TOP | WPOS_KEEP_HORZ, NULL);
             }
 
-            ui->total_files_processed_label = label_new (y++, x, "");
-            add_widget (ui->op_dlg, ui->total_files_processed_label);
+            ui->total_files_processed_label = label_new (y++, x, NULL);
+            group_add_widget (g, ui->total_files_processed_label);
 
-            ui->time_label = label_new (y++, x, "");
-            add_widget (ui->op_dlg, ui->time_label);
+            ui->time_label = label_new (y++, x, NULL);
+            group_add_widget (g, ui->time_label);
         }
     }
     else
     {
-        ui->src_file = label_new (y++, x, "");
-        add_widget (ui->op_dlg, ui->src_file);
+        ui->src_file = label_new (y++, x, NULL);
+        group_add_widget (g, ui->src_file);
 
-        ui->total_files_processed_label = label_new (y++, x, "");
-        add_widget (ui->op_dlg, ui->total_files_processed_label);
+        ui->total_files_processed_label = label_new (y++, x, NULL);
+        group_add_widget (g, ui->total_files_processed_label);
     }
 
-    add_widget (ui->op_dlg, hline_new (y++, -1, -1));
+    group_add_widget (g, hline_new (y++, -1, -1));
 
     progress_buttons[0].w = WIDGET (button_new (y, 0, progress_buttons[0].action,
                                                 progress_buttons[0].flags, progress_buttons[0].text,
@@ -904,8 +934,7 @@ file_op_context_create_ui (file_op_context_t * ctx, gboolean with_eta,
                                                     progress_buttons[2].text,
                                                     progress_button_callback));
         progress_buttons[2].len = button_get_len (BUTTON (progress_buttons[2].w));
-        send_message (progress_buttons[2].w, NULL, MSG_DESTROY, 0, NULL);
-        g_free (progress_buttons[2].w);
+        widget_destroy (progress_buttons[2].w);
     }
     progress_buttons[2].w = progress_buttons[1].w;
 
@@ -915,16 +944,19 @@ file_op_context_create_ui (file_op_context_t * ctx, gboolean with_eta,
     if (progress_buttons[3].len == -1)
         progress_buttons[3].len = button_get_len (BUTTON (progress_buttons[3].w));
 
-    add_widget (ui->op_dlg, progress_buttons[0].w);
-    add_widget (ui->op_dlg, progress_buttons[1].w);
-    add_widget (ui->op_dlg, progress_buttons[3].w);
+    group_add_widget (g, progress_buttons[0].w);
+    group_add_widget (g, progress_buttons[1].w);
+    group_add_widget (g, progress_buttons[3].w);
 
     buttons_width = 2 +
         progress_buttons[0].len + MAX (progress_buttons[1].len, progress_buttons[2].len) +
         progress_buttons[3].len;
 
     /* adjust dialog sizes  */
-    dlg_set_size (ui->op_dlg, y + 3, MAX (COLS * 2 / 3, buttons_width + 6));
+    r = w->rect;
+    r.lines = y + 3;
+    r.cols = MAX (COLS * 2 / 3, buttons_width + 6);
+    widget_set_size_rect (w, &r);
 
     place_progress_buttons (ui->op_dlg, FALSE);
 
@@ -945,7 +977,7 @@ file_op_context_destroy_ui (file_op_context_t * ctx)
         file_op_context_ui_t *ui = (file_op_context_ui_t *) ctx->ui;
 
         dlg_run_done (ui->op_dlg);
-        dlg_destroy (ui->op_dlg);
+        widget_destroy (WIDGET (ui->op_dlg));
         MC_PTR_FREE (ctx->ui);
     }
 }
@@ -960,7 +992,6 @@ file_progress_show (file_op_context_t * ctx, off_t done, off_t total,
                     const char *stalled_msg, gboolean force_update)
 {
     file_op_context_ui_t *ui;
-    char buffer[BUF_TINY];
 
     if (!verbose || ctx == NULL || ctx->ui == NULL)
         return;
@@ -979,25 +1010,24 @@ file_progress_show (file_op_context_t * ctx, off_t done, off_t total,
     if (!force_update)
         return;
 
-    if (ui->showing_eta && ctx->eta_secs > 0.5)
+    if (!ui->showing_eta || ctx->eta_secs <= 0.5)
+        label_set_text (ui->progress_file_label, stalled_msg);
+    else
     {
         char buffer2[BUF_TINY];
 
         file_eta_prepare_for_show (buffer2, ctx->eta_secs, FALSE);
         if (ctx->bps == 0)
-            g_snprintf (buffer, sizeof (buffer), "%s %s", buffer2, stalled_msg);
+            label_set_textv (ui->progress_file_label, "%s %s", buffer2, stalled_msg);
         else
         {
             char buffer3[BUF_TINY];
 
             file_bps_prepare_for_show (buffer3, ctx->bps);
-            g_snprintf (buffer, sizeof (buffer), "%s (%s) %s", buffer2, buffer3, stalled_msg);
+            label_set_textv (ui->progress_file_label, "%s (%s) %s", buffer2, buffer3, stalled_msg);
         }
-    }
-    else
-        g_snprintf (buffer, sizeof (buffer), "%s", stalled_msg);
 
-    label_set_text (ui->progress_file_label, buffer);
+    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1005,7 +1035,6 @@ file_progress_show (file_op_context_t * ctx, off_t done, off_t total,
 void
 file_progress_show_count (file_op_context_t * ctx, size_t done, size_t total)
 {
-    char buffer[BUF_TINY];
     file_op_context_ui_t *ui;
 
     if (ctx == NULL || ctx->ui == NULL)
@@ -1017,10 +1046,10 @@ file_progress_show_count (file_op_context_t * ctx, size_t done, size_t total)
         return;
 
     if (ctx->progress_totals_computed)
-        g_snprintf (buffer, sizeof (buffer), _("Files processed: %zu/%zu"), done, total);
+        label_set_textv (ui->total_files_processed_label, _("Files processed: %zu / %zu"), done,
+                         total);
     else
-        g_snprintf (buffer, sizeof (buffer), _("Files processed: %zu"), done);
-    label_set_text (ui->total_files_processed_label, buffer);
+        label_set_textv (ui->total_files_processed_label, _("Files processed: %zu"), done);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1029,7 +1058,6 @@ void
 file_progress_show_total (file_op_total_context_t * tctx, file_op_context_t * ctx,
                           uintmax_t copied_bytes, gboolean show_summary)
 {
-    char buffer[BUF_TINY];
     char buffer2[BUF_TINY];
     char buffer3[BUF_TINY];
     file_op_context_ui_t *ui;
@@ -1056,36 +1084,33 @@ file_progress_show_total (file_op_total_context_t * tctx, file_op_context_t * ct
 
     if (ui->time_label != NULL)
     {
-        struct timeval tv_current;
+        gint64 tv_current;
         char buffer4[BUF_TINY];
 
-        gettimeofday (&tv_current, NULL);
-        file_frmt_time (buffer2, tv_current.tv_sec - tctx->transfer_start.tv_sec);
+        tv_current = g_get_monotonic_time ();
+        file_frmt_time (buffer2, (tv_current - tctx->transfer_start) / G_USEC_PER_SEC);
 
         if (ctx->progress_totals_computed)
         {
             file_eta_prepare_for_show (buffer3, tctx->eta_secs, TRUE);
             if (tctx->bps == 0)
-                g_snprintf (buffer, sizeof (buffer), _("Time: %s %s"), buffer2, buffer3);
+                label_set_textv (ui->time_label, _("Time: %s %s"), buffer2, buffer3);
             else
             {
                 file_bps_prepare_for_show (buffer4, (long) tctx->bps);
-                g_snprintf (buffer, sizeof (buffer), _("Time: %s %s (%s)"), buffer2, buffer3,
-                            buffer4);
+                label_set_textv (ui->time_label, _("Time: %s %s (%s)"), buffer2, buffer3, buffer4);
             }
         }
         else
         {
             if (tctx->bps == 0)
-                g_snprintf (buffer, sizeof (buffer), _("Time: %s"), buffer2);
+                label_set_textv (ui->time_label, _("Time: %s"), buffer2);
             else
             {
                 file_bps_prepare_for_show (buffer4, (long) tctx->bps);
-                g_snprintf (buffer, sizeof (buffer), _("Time: %s (%s)"), buffer2, buffer4);
+                label_set_textv (ui->time_label, _("Time: %s (%s)"), buffer2, buffer4);
             }
         }
-
-        label_set_text (ui->time_label, buffer);
     }
 
     if (ui->total_bytes_label != NULL)
@@ -1093,14 +1118,12 @@ file_progress_show_total (file_op_total_context_t * tctx, file_op_context_t * ct
         size_trunc_len (buffer2, 5, tctx->copied_bytes, 0, panels_options.kilobyte_si);
 
         if (!ctx->progress_totals_computed)
-            g_snprintf (buffer, sizeof (buffer), _(" Total: %s "), buffer2);
+            hline_set_textv (ui->total_bytes_label, _(" Total: %s "), buffer2);
         else
         {
             size_trunc_len (buffer3, 5, ctx->progress_bytes, 0, panels_options.kilobyte_si);
-            g_snprintf (buffer, sizeof (buffer), _(" Total: %s/%s "), buffer2, buffer3);
+            hline_set_textv (ui->total_bytes_label, _(" Total: %s / %s "), buffer2, buffer3);
         }
-
-        hline_set_text (ui->total_bytes_label, buffer);
     }
 }
 
@@ -1129,8 +1152,8 @@ file_progress_show_source (file_op_context_t * ctx, const vfs_path_t * vpath)
     }
     else
     {
-        label_set_text (ui->src_file_label, "");
-        label_set_text (ui->src_file, "");
+        label_set_text (ui->src_file_label, NULL);
+        label_set_text (ui->src_file, NULL);
     }
 }
 
@@ -1153,8 +1176,8 @@ file_progress_show_target (file_op_context_t * ctx, const vfs_path_t * vpath)
     }
     else
     {
-        label_set_text (ui->tgt_file_label, "");
-        label_set_text (ui->tgt_file, "");
+        label_set_text (ui->tgt_file_label, NULL);
+        label_set_text (ui->tgt_file, NULL);
     }
 }
 
@@ -1163,9 +1186,9 @@ file_progress_show_target (file_op_context_t * ctx, const vfs_path_t * vpath)
 gboolean
 file_progress_show_deleting (file_op_context_t * ctx, const char *s, size_t * count)
 {
-    static guint64 timestamp = 0;
+    static gint64 timestamp = 0;
     /* update with 25 FPS rate */
-    static const guint64 delay = G_USEC_PER_SEC / 25;
+    static const gint64 delay = G_USEC_PER_SEC / 25;
 
     gboolean ret;
 
@@ -1274,15 +1297,15 @@ file_progress_real_query_replace (file_op_context_t * ctx, enum OperationMode mo
 /* --------------------------------------------------------------------------------------------- */
 
 char *
-file_mask_dialog (file_op_context_t * ctx, FileOperation operation,
-                  gboolean only_one,
-                  const char *format, const void *text, const char *def_text, gboolean * do_bg)
+file_mask_dialog (file_op_context_t * ctx, gboolean only_one, const char *format, const void *text,
+                  const char *def_text, gboolean * do_bg)
 {
     size_t fmd_xlen;
     vfs_path_t *vpath;
     gboolean source_easy_patterns = easy_patterns;
     char fmd_buf[BUF_MEDIUM];
-    char *dest_dir, *tmp;
+    char *dest_dir = NULL;
+    char *tmp;
     char *def_text_secure;
 
     if (ctx == NULL)
@@ -1296,7 +1319,7 @@ file_mask_dialog (file_op_context_t * ctx, FileOperation operation,
     /* filter out a possible password from def_text */
     vpath = vfs_path_from_str_flags (def_text, only_one ? VPF_NO_CANON : VPF_NONE);
     tmp = vfs_path_to_str_flags (vpath, 0, VPF_STRIP_PASSWORD);
-    vfs_path_free (vpath);
+    vfs_path_free (vpath, TRUE);
 
     if (source_easy_patterns)
         def_text_secure = strutils_glob_escape (tmp);
@@ -1334,22 +1357,23 @@ file_mask_dialog (file_op_context_t * ctx, FileOperation operation,
     }
 
     {
-        char *source_mask, *orig_mask;
+        char *source_mask = NULL;
+        char *orig_mask;
         int val;
         struct stat buf;
 
         quick_widget_t quick_widgets[] = {
             /* *INDENT-OFF* */
-            QUICK_LABELED_INPUT (fmd_buf, input_label_above,
-                                 easy_patterns ? "*" : "^(.*)$", "input-def", &source_mask,
-                                 NULL, FALSE, FALSE, INPUT_COMPLETE_FILENAMES),
+            QUICK_LABELED_INPUT (fmd_buf, input_label_above, easy_patterns ? "*" : "^(.*)$",
+                                 "input-def", &source_mask, NULL, FALSE, FALSE,
+                                 INPUT_COMPLETE_FILENAMES),
             QUICK_START_COLUMNS,
                 QUICK_SEPARATOR (FALSE),
             QUICK_NEXT_COLUMN,
                 QUICK_CHECKBOX (N_("&Using shell patterns"), &source_easy_patterns, NULL),
             QUICK_STOP_COLUMNS,
-            QUICK_LABELED_INPUT (N_("to:"), input_label_above,
-                                 def_text_secure, "input2", &dest_dir, NULL, FALSE, FALSE, INPUT_COMPLETE_FILENAMES),
+            QUICK_LABELED_INPUT (N_("to:"), input_label_above, def_text_secure, "input2", &dest_dir,
+                                 NULL, FALSE, FALSE, INPUT_COMPLETE_FILENAMES),
             QUICK_SEPARATOR (TRUE),
             QUICK_START_COLUMNS,
                 QUICK_CHECKBOX (N_("Follow &links"), &ctx->follow_links, NULL),
@@ -1368,55 +1392,56 @@ file_mask_dialog (file_op_context_t * ctx, FileOperation operation,
             /* *INDENT-ON* */
         };
 
+        WRect r = { -1, -1, 0, fmd_xlen };
+
         quick_dialog_t qdlg = {
-            -1, -1, fmd_xlen,
-            op_names[operation], "[Mask Copy/Rename]",
+            r, op_names[ctx->operation], "[Mask Copy/Rename]",
             quick_widgets, NULL, NULL
         };
 
-      ask_file_mask:
-        val = quick_dialog_skip (&qdlg, 4);
-
-        if (val == B_CANCEL)
+        while (TRUE)
         {
-            g_free (def_text_secure);
-            return NULL;
-        }
+            val = quick_dialog_skip (&qdlg, 4);
 
-        ctx->stat_func = ctx->follow_links ? mc_stat : mc_lstat;
+            if (val == B_CANCEL)
+            {
+                g_free (def_text_secure);
+                return NULL;
+            }
 
-        if (ctx->op_preserve)
-        {
-            ctx->preserve = TRUE;
-            ctx->umask_kill = 0777777;
-            ctx->preserve_uidgid = (geteuid () == 0);
-        }
-        else
-        {
-            mode_t i2;
+            ctx->stat_func = ctx->follow_links ? mc_stat : mc_lstat;
 
-            ctx->preserve = ctx->preserve_uidgid = FALSE;
-            i2 = umask (0);
-            umask (i2);
-            ctx->umask_kill = i2 ^ 0777777;
-        }
+            if (ctx->op_preserve)
+            {
+                ctx->preserve = TRUE;
+                ctx->umask_kill = 0777777;
+                ctx->preserve_uidgid = (geteuid () == 0);
+            }
+            else
+            {
+                mode_t i2;
 
-        if ((dest_dir == NULL) || (*dest_dir == '\0'))
-        {
-            g_free (def_text_secure);
-            g_free (source_mask);
-            g_free (dest_dir);
-            return NULL;
-        }
+                ctx->preserve = ctx->preserve_uidgid = FALSE;
+                i2 = umask (0);
+                umask (i2);
+                ctx->umask_kill = i2 ^ 0777777;
+            }
 
-        ctx->search_handle = mc_search_new (source_mask, NULL);
+            if (*dest_dir == '\0')
+            {
+                g_free (def_text_secure);
+                g_free (source_mask);
+                g_free (dest_dir);
+                return NULL;
+            }
+
+            ctx->search_handle = mc_search_new (source_mask, NULL);
+            if (ctx->search_handle != NULL)
+                break;
 
-        if (ctx->search_handle == NULL)
-        {
             message (D_ERROR, MSG_ERROR, _("Invalid source pattern '%s'"), source_mask);
-            g_free (dest_dir);
-            g_free (source_mask);
-            goto ask_file_mask;
+            MC_PTR_FREE (dest_dir);
+            MC_PTR_FREE (source_mask);
         }
 
         g_free (def_text_secure);
@@ -1461,7 +1486,7 @@ file_mask_dialog (file_op_context_t * ctx, FileOperation operation,
             dest_dir = g_strdup ("./");
         }
 
-        vfs_path_free (vpath);
+        vfs_path_free (vpath, TRUE);
 
         if (val == B_USER)
             *do_bg = TRUE;
