@@ -6,7 +6,7 @@
    created and destroyed.  This is required for the future vfs layer,
    it will be possible to have tree views over virtual file systems.
 
-   Copyright (C) 1994-2019
+   Copyright (C) 1994-2024
    Free Software Foundation, Inc.
 
    Written by:
@@ -14,7 +14,7 @@
    Norbert Warmuth, 1997
    Miguel de Icaza, 1996, 1999
    Slava Zanko <slavazanko@gmail.com>, 2013
-   Andrew Borodin <aborodin@vmail.ru>, 2013, 2014, 2016
+   Andrew Borodin <aborodin@vmail.ru>, 2013-2022
 
    This file is part of the Midnight Commander.
 
@@ -56,16 +56,17 @@
 #include "lib/event.h"          /* mc_event_raise() */
 
 #include "src/setup.h"          /* confirm_delete, panels_options */
-#include "src/keybind-defaults.h"
+#include "src/keymap.h"
 #include "src/history.h"
 
 #include "dir.h"
-#include "midnight.h"           /* the_menubar */
+#include "filemanager.h"        /* the_menubar */
 #include "file.h"               /* copy_dir_dir(), move_dir_dir(), erase_dir() */
 #include "layout.h"             /* command_prompt */
 #include "treestore.h"
 #include "cmd.h"
 #include "filegui.h"
+#include "cd.h"                 /* cd_error_message() */
 
 #include "tree.h"
 
@@ -82,12 +83,8 @@ gboolean xtree_mode = FALSE;
 
 /*** file scope macro definitions ****************************************************************/
 
-#define tlines(t) (t->is_panel ? WIDGET (t)->lines - 2 - \
-                    (panels_options.show_mini_info ? 2 : 0) : WIDGET (t)->lines)
-
-/* Use the color of the parent widget for the unselected entries */
-#define TREE_NORMALC(h) (h->color[DLG_COLOR_NORMAL])
-#define TREE_CURRENTC(h) (h->color[DLG_COLOR_FOCUS])
+#define tlines(t) (t->is_panel ? WIDGET (t)->rect.lines - 2 - \
+                    (panels_options.show_mini_info ? 2 : 0) : WIDGET (t)->rect.lines)
 
 /*** file scope type declarations ****************************************************************/
 
@@ -96,7 +93,7 @@ struct WTree
     Widget widget;
     struct TreeStore *store;
     tree_entry *selected_ptr;   /* The selected directory */
-    char search_buffer[MC_MAXFILENAMELEN];      /* Current search string */
+    GString *search_buffer;     /* Current search string */
     tree_entry **tree_shown;    /* Entries currently on screen */
     gboolean is_panel;          /* panel or plain widget flag */
     gboolean searching;         /* Are we on searching mode? */
@@ -104,6 +101,10 @@ struct WTree
                                    shown and the selected */
 };
 
+/*** forward declarations (file scope functions) *************************************************/
+
+static void tree_rescan (void *data);
+
 /*** file scope variables ************************************************************************/
 
 /* Specifies the display mode: 1d or 2d */
@@ -113,10 +114,6 @@ static gboolean tree_navigation_flag = FALSE;
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
-static void tree_rescan (void *data);
-
-/* --------------------------------------------------------------------------------------------- */
-
 static tree_entry *
 back_ptr (tree_entry * ptr, int *count)
 {
@@ -199,6 +196,7 @@ tree_destroy (WTree * tree)
     save_tree (tree);
 
     MC_PTR_FREE (tree->tree_shown);
+    g_string_free (tree->search_buffer, TRUE);
     tree->selected_ptr = NULL;
 }
 
@@ -215,7 +213,7 @@ load_tree (WTree * tree)
     tree->selected_ptr = tree->store->tree_first;
     vpath = vfs_path_from_str (mc_config_get_home_dir ());
     tree_chdir (tree, vpath);
-    vfs_path_free (vpath);
+    vfs_path_free (vpath, TRUE);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -240,20 +238,22 @@ tree_show_mini_info (WTree * tree, int tree_lines, int tree_cols)
     {
         /* Show search string */
         tty_setcolor (INPUT_COLOR);
-        tty_draw_hline (w->y + line, w->x + 1, ' ', tree_cols);
-        widget_move (w, line, 1);
+        tty_draw_hline (w->rect.y + line, w->rect.x + 1, ' ', tree_cols);
+        widget_gotoyx (w, line, 1);
         tty_print_char (PATH_SEP);
-        tty_print_string (str_fit_to_term (tree->search_buffer, tree_cols - 2, J_LEFT_FIT));
+        tty_print_string (str_fit_to_term (tree->search_buffer->str, tree_cols - 2, J_LEFT_FIT));
         tty_print_char (' ');
     }
     else
     {
         /* Show full name of selected directory */
-        WDialog *h = w->owner;
 
-        tty_setcolor (tree->is_panel ? NORMAL_COLOR : TREE_NORMALC (h));
-        tty_draw_hline (w->y + line, w->x + 1, ' ', tree_cols);
-        widget_move (w, line, 1);
+        const int *colors;
+
+        colors = widget_get_colors (w);
+        tty_setcolor (tree->is_panel ? NORMAL_COLOR : colors[DLG_COLOR_NORMAL]);
+        tty_draw_hline (w->rect.y + line, w->rect.x + 1, ' ', tree_cols);
+        widget_gotoyx (w, line, 1);
         tty_print_string (str_fit_to_term
                           (vfs_path_as_str (tree->selected_ptr->name), tree_cols, J_LEFT_FIT));
     }
@@ -265,7 +265,6 @@ static void
 show_tree (WTree * tree)
 {
     Widget *w = WIDGET (tree);
-    WDialog *h = w->owner;
     tree_entry *current;
     int i, j;
     int topsublevel = 0;
@@ -274,9 +273,9 @@ show_tree (WTree * tree)
 
     /* Initialize */
     tree_lines = tlines (tree);
-    tree_cols = w->cols;
+    tree_cols = w->rect.cols;
 
-    widget_move (w, y, x);
+    widget_gotoyx (w, y, x);
     if (tree->is_panel)
     {
         tree_cols -= 2;
@@ -335,19 +334,30 @@ show_tree (WTree * tree)
     /* Loop for every line */
     for (i = 0; i < tree_lines; i++)
     {
-        tty_setcolor (tree->is_panel ? NORMAL_COLOR : TREE_NORMALC (h));
+        const int *colors;
+
+        colors = widget_get_colors (w);
+        tty_setcolor (tree->is_panel ? NORMAL_COLOR : colors[DLG_COLOR_NORMAL]);
 
         /* Move to the beginning of the line */
-        tty_draw_hline (w->y + y + i, w->x + x, ' ', tree_cols);
+        tty_draw_hline (w->rect.y + y + i, w->rect.x + x, ' ', tree_cols);
 
         if (current == NULL)
             continue;
 
         if (tree->is_panel)
-            tty_setcolor (widget_get_state (w, WST_FOCUSED) && current == tree->selected_ptr
-                          ? SELECTED_COLOR : NORMAL_COLOR);
+        {
+            gboolean selected;
+
+            selected = widget_get_state (w, WST_FOCUSED) && current == tree->selected_ptr;
+            tty_setcolor (selected ? SELECTED_COLOR : NORMAL_COLOR);
+        }
         else
-            tty_setcolor (current == tree->selected_ptr ? TREE_CURRENTC (h) : TREE_NORMALC (h));
+        {
+            int idx = current == tree->selected_ptr ? DLG_COLOR_FOCUS : DLG_COLOR_NORMAL;
+
+            tty_setcolor (colors[idx]);
+        }
 
         tree->tree_shown[i] = current;
         if (current->sublevel == topsublevel)
@@ -577,24 +587,25 @@ tree_chdir_sel (WTree * tree)
 {
     if (tree->is_panel)
     {
-        change_panel ();
+        WPanel *p;
 
-        if (do_cd (tree->selected_ptr->name, cd_exact))
-            select_item (current_panel);
+        p = change_panel ();
+
+        if (panel_cd (p, tree->selected_ptr->name, cd_exact))
+            select_item (p);
         else
-            message (D_ERROR, MSG_ERROR, _("Cannot chdir to \"%s\"\n%s"),
-                     vfs_path_as_str (tree->selected_ptr->name), unix_error_string (errno));
+            cd_error_message (vfs_path_as_str (tree->selected_ptr->name));
 
-        widget_redraw (WIDGET (current_panel));
-        change_panel ();
+        widget_draw (WIDGET (p));
+        (void) change_panel ();
         show_tree (tree);
     }
     else
     {
-        WDialog *h = WIDGET (tree)->owner;
+        WDialog *h = DIALOG (WIDGET (tree)->owner);
 
         h->ret_value = B_ENTER;
-        dlg_stop (h);
+        dlg_close (h);
     }
 }
 
@@ -610,19 +621,15 @@ maybe_chdir (WTree * tree)
 /* --------------------------------------------------------------------------------------------- */
 /** Search tree for text */
 
-static int
-search_tree (WTree * tree, char *text)
+static gboolean
+search_tree (WTree * tree, const GString * text)
 {
-    tree_entry *current;
-    size_t len;
+    tree_entry *current = tree->selected_ptr;
     gboolean wrapped = FALSE;
     gboolean found = FALSE;
 
-    len = strlen (text);
-    current = tree->selected_ptr;
-
     while (!found && (!wrapped || current != tree->selected_ptr))
-        if (strncmp (current->subname, text, len) == 0)
+        if (strncmp (current->subname, text->str, text->len) == 0)
         {
             tree->selected_ptr = current;
             found = TRUE;
@@ -648,19 +655,15 @@ search_tree (WTree * tree, char *text)
 static void
 tree_do_search (WTree * tree, int key)
 {
-    size_t l;
+    /* TODO: support multi-byte characters, see do_search() in panel.c */
 
-    l = strlen (tree->search_buffer);
-    if ((l != 0) && (key == KEY_BACKSPACE))
-        tree->search_buffer[--l] = '\0';
-    else if (key && l < sizeof (tree->search_buffer) - 1)
-    {
-        tree->search_buffer[l] = key;
-        tree->search_buffer[++l] = '\0';
-    }
+    if (tree->search_buffer->len != 0 && key == KEY_BACKSPACE)
+        g_string_set_size (tree->search_buffer, tree->search_buffer->len - 1);
+    else if (key != 0)
+        g_string_append_c (tree->search_buffer, (gchar) key);
 
     if (!search_tree (tree, tree->search_buffer))
-        tree->search_buffer[--l] = '\0';
+        g_string_set_size (tree->search_buffer, tree->search_buffer->len - 1);
 
     show_tree (tree);
     maybe_chdir (tree);
@@ -686,7 +689,7 @@ tree_rescan (void *data)
         ret = mc_chdir (old_vpath);
         (void) ret;
     }
-    vfs_path_free (old_vpath);
+    vfs_path_free (old_vpath, TRUE);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -742,10 +745,6 @@ tree_move (WTree * tree, const char *default_dest)
 {
     char msg[BUF_MEDIUM];
     char *dest;
-    struct stat buf;
-    file_op_context_t *ctx;
-    file_op_total_context_t *tctx;
-    vfs_path_t *dest_vpath = NULL;
 
     if (tree->selected_ptr == NULL)
         return;
@@ -756,33 +755,34 @@ tree_move (WTree * tree, const char *default_dest)
         input_expand_dialog (Q_ ("DialogTitle|Move"), msg, MC_HISTORY_FM_TREE_MOVE, default_dest,
                              INPUT_COMPLETE_FILENAMES | INPUT_COMPLETE_CD);
 
-    if (dest == NULL || *dest == '\0')
-        goto ret;
-
-    dest_vpath = vfs_path_from_str (dest);
-
-    if (mc_stat (dest_vpath, &buf))
+    if (dest != NULL && *dest != '\0')
     {
-        message (D_ERROR, MSG_ERROR, _("Cannot stat the destination\n%s"),
-                 unix_error_string (errno));
-        goto ret;
+        vfs_path_t *dest_vpath;
+        struct stat buf;
+
+        dest_vpath = vfs_path_from_str (dest);
+
+        if (mc_stat (dest_vpath, &buf))
+            message (D_ERROR, MSG_ERROR, _("Cannot stat the destination\n%s"),
+                     unix_error_string (errno));
+        else if (!S_ISDIR (buf.st_mode))
+            file_error (TRUE, _("Destination \"%s\" must be a directory\n%s"), dest);
+        else
+        {
+            file_op_context_t *ctx;
+            file_op_total_context_t *tctx;
+
+            ctx = file_op_context_new (OP_MOVE);
+            tctx = file_op_total_context_new ();
+            file_op_context_create_ui (ctx, FALSE, FILEGUI_DIALOG_ONE_ITEM);
+            move_dir_dir (tctx, ctx, vfs_path_as_str (tree->selected_ptr->name), dest);
+            file_op_total_context_destroy (tctx);
+            file_op_context_destroy (ctx);
+        }
+
+        vfs_path_free (dest_vpath, TRUE);
     }
 
-    if (!S_ISDIR (buf.st_mode))
-    {
-        file_error (TRUE, _("Destination \"%s\" must be a directory\n%s"), dest);
-        goto ret;
-    }
-
-    ctx = file_op_context_new (OP_MOVE);
-    tctx = file_op_total_context_new ();
-    file_op_context_create_ui (ctx, FALSE, FILEGUI_DIALOG_ONE_ITEM);
-    move_dir_dir (tctx, ctx, vfs_path_as_str (tree->selected_ptr->name), dest);
-    file_op_total_context_destroy (tctx);
-    file_op_context_destroy (ctx);
-
-  ret:
-    vfs_path_free (dest_vpath);
     g_free (dest);
 }
 
@@ -962,7 +962,7 @@ tree_start_search (WTree * tree)
     else
     {
         tree->searching = TRUE;
-        tree->search_buffer[0] = '\0';
+        g_string_set_size (tree->search_buffer, 0);
     }
 }
 
@@ -971,15 +971,26 @@ tree_start_search (WTree * tree)
 static void
 tree_toggle_navig (WTree * tree)
 {
+    Widget *w = WIDGET (tree);
     WButtonBar *b;
 
     tree_navigation_flag = !tree_navigation_flag;
 
-    b = find_buttonbar (WIDGET (tree)->owner);
+    b = buttonbar_find (DIALOG (w->owner));
     buttonbar_set_label (b, 4,
                          tree_navigation_flag ? Q_ ("ButtonBar|Static") : Q_ ("ButtonBar|Dynamc"),
-                         tree_map, WIDGET (tree));
-    widget_redraw (WIDGET (b));
+                         w->keymap, w);
+    widget_draw (WIDGET (b));
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static void
+tree_help (void)
+{
+    ev_help_t event_data = { NULL, "[Directory Tree]" };
+
+    mc_event_raise (MCEVENT_GROUP_CORE, "help", &event_data);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -995,10 +1006,7 @@ tree_execute_cmd (WTree * tree, long command)
     switch (command)
     {
     case CK_Help:
-        {
-            ev_help_t event_data = { NULL, "[Directory Tree]" };
-            mc_event_raise (MCEVENT_GROUP_CORE, "help", &event_data);
-        }
+        tree_help ();
         break;
     case CK_Forget:
         tree_forget (tree);
@@ -1044,7 +1052,7 @@ tree_execute_cmd (WTree * tree, long command)
         break;
     case CK_Quit:
         if (!tree->is_panel)
-            dlg_stop (WIDGET (tree)->owner);
+            dlg_close (DIALOG (WIDGET (tree)->owner));
         return res;
     default:
         res = MSG_NOT_HANDLED;
@@ -1060,7 +1068,7 @@ tree_execute_cmd (WTree * tree, long command)
 static cb_ret_t
 tree_key (WTree * tree, int key)
 {
-    size_t i;
+    long command;
 
     if (is_abort_char (key))
     {
@@ -1082,18 +1090,19 @@ tree_key (WTree * tree, int key)
         return MSG_HANDLED;
     }
 
-    for (i = 0; tree_map[i].key != 0; i++)
-        if (key == tree_map[i].key)
-            switch (tree_map[i].command)
-            {
-            case CK_Left:
-                return tree_move_left (tree) ? MSG_HANDLED : MSG_NOT_HANDLED;
-            case CK_Right:
-                return tree_move_right (tree) ? MSG_HANDLED : MSG_NOT_HANDLED;
-            default:
-                tree_execute_cmd (tree, tree_map[i].command);
-                return MSG_HANDLED;
-            }
+    command = widget_lookup_key (WIDGET (tree), key);
+    switch (command)
+    {
+    case CK_IgnoreKey:
+        break;
+    case CK_Left:
+        return tree_move_left (tree) ? MSG_HANDLED : MSG_NOT_HANDLED;
+    case CK_Right:
+        return tree_move_right (tree) ? MSG_HANDLED : MSG_NOT_HANDLED;
+    default:
+        tree_execute_cmd (tree, command);
+        return MSG_HANDLED;
+    }
 
     /* Do not eat characters not meant for the tree below ' ' (e.g. C-l). */
     if (!command_prompt && ((key >= ' ' && key <= 255) || key == KEY_BACKSPACE))
@@ -1122,21 +1131,21 @@ tree_frame (WDialog * h, WTree * tree)
         const char *title = _("Directory tree");
         const int len = str_term_width1 (title);
 
-        tty_draw_box (w->y, w->x, w->lines, w->cols, FALSE);
+        tty_draw_box (w->rect.y, w->rect.x, w->rect.lines, w->rect.cols, FALSE);
 
-        widget_move (w, 0, (w->cols - len - 2) / 2);
+        widget_gotoyx (w, 0, (w->rect.cols - len - 2) / 2);
         tty_printf (" %s ", title);
 
         if (panels_options.show_mini_info)
         {
             int y;
 
-            y = w->lines - 3;
-            widget_move (w, y, 0);
+            y = w->rect.lines - 3;
+            widget_gotoyx (w, y, 0);
             tty_print_alt_char (ACS_LTEE, FALSE);
-            widget_move (w, y, w->cols - 1);
+            widget_gotoyx (w, y, w->rect.cols - 1);
             tty_print_alt_char (ACS_RTEE, FALSE);
-            tty_draw_hline (w->y + y, w->x + 1, ACS_HLINE, w->cols - 2);
+            tty_draw_hline (w->rect.y + y, w->rect.x + 1, ACS_HLINE, w->rect.cols - 2);
         }
     }
 }
@@ -1147,7 +1156,7 @@ static cb_ret_t
 tree_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
     WTree *tree = (WTree *) w;
-    WDialog *h = w->owner;
+    WDialog *h = DIALOG (w->owner);
     WButtonBar *b;
 
     switch (msg)
@@ -1157,27 +1166,27 @@ tree_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *da
         show_tree (tree);
         if (widget_get_state (w, WST_FOCUSED))
         {
-            b = find_buttonbar (h);
-            widget_redraw (WIDGET (b));
+            b = buttonbar_find (h);
+            widget_draw (WIDGET (b));
         }
         return MSG_HANDLED;
 
     case MSG_FOCUS:
-        b = find_buttonbar (h);
-        buttonbar_set_label (b, 1, Q_ ("ButtonBar|Help"), tree_map, w);
-        buttonbar_set_label (b, 2, Q_ ("ButtonBar|Rescan"), tree_map, w);
-        buttonbar_set_label (b, 3, Q_ ("ButtonBar|Forget"), tree_map, w);
+        b = buttonbar_find (h);
+        buttonbar_set_label (b, 1, Q_ ("ButtonBar|Help"), w->keymap, w);
+        buttonbar_set_label (b, 2, Q_ ("ButtonBar|Rescan"), w->keymap, w);
+        buttonbar_set_label (b, 3, Q_ ("ButtonBar|Forget"), w->keymap, w);
         buttonbar_set_label (b, 4, tree_navigation_flag ? Q_ ("ButtonBar|Static")
-                             : Q_ ("ButtonBar|Dynamc"), tree_map, w);
-        buttonbar_set_label (b, 5, Q_ ("ButtonBar|Copy"), tree_map, w);
-        buttonbar_set_label (b, 6, Q_ ("ButtonBar|RenMov"), tree_map, w);
+                             : Q_ ("ButtonBar|Dynamc"), w->keymap, w);
+        buttonbar_set_label (b, 5, Q_ ("ButtonBar|Copy"), w->keymap, w);
+        buttonbar_set_label (b, 6, Q_ ("ButtonBar|RenMov"), w->keymap, w);
 #if 0
         /* FIXME: mkdir is currently defunct */
-        buttonbar_set_label (b, 7, Q_ ("ButtonBar|Mkdir"), tree_map, w);
+        buttonbar_set_label (b, 7, Q_ ("ButtonBar|Mkdir"), w->keymap, w);
 #else
         buttonbar_clear_label (b, 7, w);
 #endif
-        buttonbar_set_label (b, 8, Q_ ("ButtonBar|Rmdir"), tree_map, w);
+        buttonbar_set_label (b, 8, Q_ ("ButtonBar|Rmdir"), w->keymap, w);
 
         return MSG_HANDLED;
 
@@ -1220,13 +1229,13 @@ tree_mouse_callback (Widget * w, mouse_msg_t msg, mouse_event_t * event)
     {
     case MSG_MOUSE_DOWN:
         /* rest of the upper frame - call menu */
-        if (tree->is_panel && event->y == WIDGET (w->owner)->y)
+        if (tree->is_panel && event->y == WIDGET (w->owner)->rect.y)
         {
             /* return MOU_UNHANDLED */
             event->result.abort = TRUE;
         }
         else if (!widget_get_state (w, WST_FOCUSED))
-            change_panel ();
+            (void) change_panel ();
         break;
 
     case MSG_MOUSE_CLICK:
@@ -1275,22 +1284,25 @@ tree_mouse_callback (Widget * w, mouse_msg_t msg, mouse_event_t * event)
 WTree *
 tree_new (int y, int x, int lines, int cols, gboolean is_panel)
 {
+    WRect r = { y, x, lines, cols };
     WTree *tree;
     Widget *w;
 
     tree = g_new (WTree, 1);
-    w = WIDGET (tree);
 
-    widget_init (w, y, x, lines, cols, tree_callback, tree_mouse_callback);
+    w = WIDGET (tree);
+    widget_init (w, &r, tree_callback, tree_mouse_callback);
     w->options |= WOP_SELECTABLE | WOP_TOP_SELECT;
+    w->keymap = tree_map;
+
     tree->is_panel = is_panel;
     tree->selected_ptr = NULL;
 
     tree->store = tree_store_get ();
     tree_store_add_entry_remove_hook (remove_callback, tree);
     tree->tree_shown = NULL;
-    tree->search_buffer[0] = '\0';
-    tree->topdiff = w->lines / 2;
+    tree->search_buffer = g_string_sized_new (MC_MAXPATHLEN);
+    tree->topdiff = w->rect.lines / 2;
     tree->searching = FALSE;
 
     load_tree (tree);
@@ -1332,9 +1344,9 @@ sync_tree (const vfs_path_t * vpath)
 /* --------------------------------------------------------------------------------------------- */
 
 WTree *
-find_tree (WDialog * h)
+find_tree (const WDialog * h)
 {
-    return (WTree *) find_widget_type (h, tree_callback);
+    return (WTree *) widget_find_by_type (CONST_WIDGET (h), tree_callback);
 }
 
 /* --------------------------------------------------------------------------------------------- */
