@@ -1,7 +1,7 @@
 /*
    Init VFS plugins.
 
-   Copyright (C) 2011-2021
+   Copyright (C) 2011-2024
    Free Software Foundation, Inc.
 
    Written by:
@@ -46,8 +46,8 @@
 #include "extfs/extfs.h"
 #endif
 
-#ifdef ENABLE_VFS_FISH
-#include "fish/fish.h"
+#ifdef ENABLE_VFS_SHELL
+#include "shell/shell.h"
 #endif
 
 #ifdef ENABLE_VFS_FTP
@@ -55,17 +55,13 @@
 #endif
 
 #ifdef ENABLE_VFS_SFTP
-#include "sftpfs/init.h"
+#include "sftpfs/sftpfs.h"
 #endif
 
 #ifdef ENABLE_VFS_SFS
 #include "sfs/sfs.h"
 #endif
 
-#ifdef ENABLE_VFS_SMB
-#include "smbfs/smbfs.h"
-#endif
-
 #ifdef ENABLE_VFS_TAR
 #include "tar/tar.h"
 #endif
@@ -119,13 +115,9 @@ vfs_plugins_init (void)
 #ifdef ENABLE_VFS_SFTP
     vfs_init_sftpfs ();
 #endif /* ENABLE_VFS_SFTP */
-#ifdef ENABLE_VFS_FISH
-    vfs_init_fish ();
-#endif /* ENABLE_VFS_FISH */
-#ifdef ENABLE_VFS_SMB
-    vfs_init_smbfs ();
-#endif /* ENABLE_VFS_SMB */
-
+#ifdef ENABLE_VFS_SHELL
+    vfs_init_shell ();
+#endif /* ENABLE_VFS_SHELL */
 }
 
 /* --------------------------------------------------------------------------------------------- */
