@@ -1,7 +1,7 @@
 /* Virtual File System: SFTP file system.
    The internal functions: files
 
-   Copyright (C) 2011-2019
+   Copyright (C) 2011-2024
    Free Software Foundation, Inc.
 
    Written by:
@@ -26,7 +26,8 @@
 
 #include <config.h>
 
-#include <errno.h>
+#include <errno.h>              /* ENOENT, EACCES */
+
 #include <libssh2.h>
 #include <libssh2_sftp.h>
 
@@ -52,6 +53,8 @@ typedef struct
     mode_t mode;
 } sftpfs_file_handler_t;
 
+/*** forward declarations (file scope functions) *************************************************/
+
 /*** file scope variables ************************************************************************/
 
 /* --------------------------------------------------------------------------------------------- */
@@ -131,11 +134,12 @@ sftpfs_open_file (vfs_file_handler_t * fh, int flags, mode_t mode, GError ** mce
     sftpfs_file_handler_t *file = SFTP_FILE_HANDLER (fh);
     sftpfs_super_t *super = SFTP_SUPER (fh->ino->super);
     char *name;
+    const GString *fixfname;
 
     (void) mode;
     mc_return_val_if_error (mcerror, FALSE);
 
-    name = vfs_s_fullpath (sftpfs_class, fh->ino);
+    name = vfs_s_fullpath (vfs_sftpfs_ops, fh->ino);
     if (name == NULL)
         return FALSE;
 
@@ -156,17 +160,15 @@ sftpfs_open_file (vfs_file_handler_t * fh, int flags, mode_t mode, GError ** mce
     else
         sftp_open_flags = LIBSSH2_FXF_READ;
 
+    fixfname = sftpfs_fix_filename (name);
+
     while (TRUE)
     {
-        const char *fixfname;
-        unsigned int fixfname_len = 0;
         int libssh_errno;
 
-        fixfname = sftpfs_fix_filename (name, &fixfname_len);
-
         file->handle =
-            libssh2_sftp_open_ex (super->sftp_session, fixfname, fixfname_len, sftp_open_flags,
-                                  sftp_open_mode, LIBSSH2_SFTP_OPENFILE);
+            libssh2_sftp_open_ex (super->sftp_session, fixfname->str, fixfname->len,
+                                  sftp_open_flags, sftp_open_mode, LIBSSH2_SFTP_OPENFILE);
         if (file->handle != NULL)
             break;
 
@@ -175,7 +177,6 @@ sftpfs_open_file (vfs_file_handler_t * fh, int flags, mode_t mode, GError ** mce
         {
             sftpfs_ssherror_to_gliberror (super, libssh_errno, mcerror);
             g_free (name);
-            g_free (file);
             return FALSE;
         }
     }
