@@ -2,7 +2,7 @@
    Internal file viewer for the Midnight Commander
    Common finctions (used from some other mcviewer functions)
 
-   Copyright (C) 1994-2019
+   Copyright (C) 1994-2024
    Free Software Foundation, Inc.
 
    Written by:
@@ -14,7 +14,7 @@
    Pavel Machek, 1998
    Roland Illig <roland.illig@gmx.de>, 2004, 2005
    Slava Zanko <slavazanko@google.com>, 2009, 2013
-   Andrew Borodin <aborodin@vmail.ru>, 2009, 2013, 2014
+   Andrew Borodin <aborodin@vmail.ru>, 2009-2022
    Ilia Maslakov <il.smind@gmail.com>, 2009
 
    This file is part of the Midnight Commander.
@@ -217,17 +217,20 @@ mcview_done (WView * view)
 
     /* Free memory used by the viewer */
     /* view->widget needs no destructor */
-    vfs_path_free (view->filename_vpath);
+    vfs_path_free (view->filename_vpath, TRUE);
     view->filename_vpath = NULL;
-    vfs_path_free (view->workdir_vpath);
+    vfs_path_free (view->workdir_vpath, TRUE);
     view->workdir_vpath = NULL;
     MC_PTR_FREE (view->command);
 
     mcview_close_datasource (view);
     /* the growing buffer is freed with the datasource */
 
-    coord_cache_free (view->coord_cache);
-    view->coord_cache = NULL;
+    if (view->coord_cache != NULL)
+    {
+        g_ptr_array_free (view->coord_cache, TRUE);
+        view->coord_cache = NULL;
+    }
 
     if (view->converter == INVALID_CONV)
         view->converter = str_cnv_from_term;
@@ -238,10 +241,9 @@ mcview_done (WView * view)
         view->converter = str_cnv_from_term;
     }
 
-    mc_search_free (view->search);
+    mcview_search_deinit (view);
     view->search = NULL;
-    MC_PTR_FREE (view->last_search_string);
-    mcview_nroff_seq_free (&view->search_nroff_seq);
+    view->last_search_string = NULL;
     mcview_hexedit_free_change_list (view);
 
     if (mc_global.mc_run_mode == MC_RUN_VIEWER && view->dir != NULL)
@@ -257,10 +259,10 @@ mcview_done (WView * view)
 
 /* --------------------------------------------------------------------------------------------- */
 
+#ifdef HAVE_CHARSET
 void
 mcview_set_codeset (WView * view)
 {
-#ifdef HAVE_CHARSET
     const char *cp_id = NULL;
 
     view->utf8 = TRUE;
@@ -280,21 +282,17 @@ mcview_set_codeset (WView * view)
         view->utf8 = (gboolean) str_isutf8 (cp_id);
         view->dpy_wrap_dirty = TRUE;
     }
-#else
-    (void) view;
-#endif
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-#ifdef HAVE_CHARSET
 void
 mcview_select_encoding (WView * view)
 {
     if (do_select_codepage ())
         mcview_set_codeset (view);
 }
-#endif
+#endif /* HAVE_CHARSET */
 
 /* --------------------------------------------------------------------------------------------- */
 
@@ -379,12 +377,14 @@ mcview_eol (WView * view, off_t current)
 char *
 mcview_get_title (const WDialog * h, size_t len)
 {
-    const WView *view = (const WView *) find_widget_type (h, mcview_callback);
-    const char *modified = view->hexedit_mode && (view->change_list != NULL) ? "(*) " : "    ";
+    const WView *view;
+    const char *modified;
     const char *file_label;
     const char *view_filename;
     char *ret_str;
 
+    view = (const WView *) widget_find_by_type (CONST_WIDGET (h), mcview_callback);
+    modified = view->hexedit_mode && (view->change_list != NULL) ? "(*) " : "    ";
     view_filename = vfs_path_as_str (view->filename_vpath);
 
     len -= 4;
@@ -401,12 +401,10 @@ mcview_get_title (const WDialog * h, size_t len)
 int
 mcview_calc_percent (WView * view, off_t p)
 {
-    const screen_dimen right = view->status_area.left + view->status_area.width;
-    const screen_dimen height = view->status_area.height;
     off_t filesize;
     int percent;
 
-    if (height < 1 || right < 4)
+    if (view->status_area.cols < 1 || (view->status_area.x + view->status_area.cols) < 4)
         return (-1);
     if (mcview_may_still_grow (view))
         return (-1);
