@@ -1,7 +1,7 @@
 /*
    lib/vfs - vfs_path_t charset recode functions
 
-   Copyright (C) 2011-2019
+   Copyright (C) 2011-2024
    Free Software Foundation, Inc.
 
    Written by:
@@ -139,22 +139,22 @@ START_PARAMETRIZED_TEST (test_path_recode, test_path_recode_ds)
 {
     /* given */
     vfs_path_t *vpath;
-    const vfs_path_element_t *element;
+    const char *element_path;
     const char *vpath_str;
 
     test_init_vfs (data->input_codepage);
 
     /* when */
     vpath = vfs_path_from_str (data->input_path);
-    element = vfs_path_get_by_index (vpath, -1);
+    element_path = vfs_path_get_last_path_str (vpath);
 
     /* then */
     vpath_str = vfs_path_as_str (vpath);
     mctest_assert_ptr_ne (vpath, NULL);
-    mctest_assert_str_eq (element->path, data->expected_element_path);
+    mctest_assert_str_eq (element_path, data->expected_element_path);
     mctest_assert_str_eq (vpath_str, data->expected_recoded_path);
 
-    vfs_path_free (vpath);
+    vfs_path_free (vpath, TRUE);
     test_deinit_vfs ();
 }
 /* *INDENT-OFF* */
@@ -244,7 +244,7 @@ START_PARAMETRIZED_TEST (test_path_to_str_flags, test_path_to_str_flags_ds)
 
     test_init_vfs ("UTF-8");
 
-    vfs_init_class (&vfs_test_ops1, "testfs1", VFS_NOLINKS | VFS_REMOTE, "test1");
+    vfs_init_class (&vfs_test_ops1, "testfs1", VFSF_NOLINKS | VFSF_REMOTE, "test1");
     vfs_register_class (&vfs_test_ops1);
 
     /* when */
@@ -256,7 +256,7 @@ START_PARAMETRIZED_TEST (test_path_to_str_flags, test_path_to_str_flags_ds)
     mctest_assert_str_eq (str_path, data->expected_path);
 
     g_free (str_path);
-    vfs_path_free (vpath);
+    vfs_path_free (vpath, TRUE);
     test_deinit_vfs ();
 }
 /* *INDENT-OFF* */
@@ -268,11 +268,9 @@ END_PARAMETRIZED_TEST
 int
 main (void)
 {
-    int number_failed;
+    TCase *tc_core;
 
-    Suite *s = suite_create (TEST_SUITE_NAME);
-    TCase *tc_core = tcase_create ("Core");
-    SRunner *sr;
+    tc_core = tcase_create ("Core");
 
     tcase_add_checked_fixture (tc_core, setup, teardown);
 
@@ -281,13 +279,7 @@ main (void)
     mctest_add_parameterized_test (tc_core, test_path_to_str_flags, test_path_to_str_flags_ds);
     /* *********************************** */
 
-    suite_add_tcase (s, tc_core);
-    sr = srunner_create (s);
-    srunner_set_log (sr, "path_recode.log");
-    srunner_run_all (sr, CK_ENV);
-    number_failed = srunner_ntests_failed (sr);
-    srunner_free (sr);
-    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
+    return mctest_run_all (tc_core);
 }
 
 /* --------------------------------------------------------------------------------------------- */
